/***********************************************************************
 * File: aggregate.c
 * 	(Extracted from
 *	   "$Id: aggregate.c,v 1.13 2009/06/21 15:37:56 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C source file implements `aggregate()' API.
 *
 * Copyright (C) 2009 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#include "aggregate.h" /* definitions of `aggregate' API-s */

/* type definition of the result buffer slot */
typedef struct result_t {
	size_t	size;	/* size of the allocated result buffer		*/
	char	*bufp;	/* pointer to the allocated result buffer	*/
} Result_t;

/* definition of signs for `Number_t' */
typedef enum sign_e {
	  Sign_e_positive	/* sign is positive. */
	, Sign_e_negative	/* sign is negative. */
}	Sign_e;

/* type definition of the number for processing	*/
typedef struct number_t {
	const char	*ip;	/* pointer to integer part			*/
	const char	*dp;	/* pointer to decimal point			*/
	const char	*ep;	/* pointer to end of numeral string	*/
	Sign_e		sign;	/* sign of number					*/
} Number_t;

/* the Number_t structure `x' means zero or not? */
static int is_zero(Number_t	*x)
{
	int	result	= (((x->ip == x->dp) && x->ip == x->ep)? 1: 0);

	return	result;
}

/* check the characters is sequence of digits? */
static int is_sequence_of_digits(const char	*sp, const char	*ep)
{
	while (sp < ep) {
		if (!isdigit(*sp)) {
			return	0; /* this characters isn't sequence of digits	*/
		}
		++sp;
	}

	return	!0;	/* this characters is a sequence of digits	*/
}

/* make a Number_t structure as the numeral string */
/*@null@*/ static Number_t *make_number(const char			*numeral_str
									, /*@out@*/	Number_t	*number
									)
{
	const char	*p = NULL;

	/* test: arguments are invalid? */
	if ((number == NULL) || numeral_str == NULL) {
		return	NULL; /* Error: invalid arguments */
	}

	/* initialize `number' structure */
	memset(number, (int)'\0', sizeof(*number));

	/* set pointers to `decimal point' and 'end of fraction part' */
	p	= numeral_str + strlen(numeral_str);
	if ((number->dp = strchr(numeral_str, '.')) == NULL) {
		while ((numeral_str < p) && *(p - 1) == ' ') {	--p;	}
		number->dp	= (char *)p;
	}
	while ((number->dp < p) && *(p - 1) == ' ') {	--p;	}
	while ((number->dp < p) && *(p - 1) == '0') {	--p;	}
	number->ep	= ((number->dp == (p - 1))? number->dp:	(char *)p);

	/* test: fraction part is filled with digits? */
	if (number->dp != number->ep) {
		if (is_sequence_of_digits((number->dp + 1), number->ep) == 0) {
			return	NULL;	/* Error: invalid arguments */
		}
	}

	/* set sign and pointer to `start of integer part' */
	p	= numeral_str;
	while ((p < number->dp) && *p == ' ') {	++p;	}
	number->sign = (((p == number->dp) || *p != '-')?	Sign_e_positive
					:									Sign_e_negative
					);
	if ((*p == '+') || *p == '-') {	++p;	}
	if (p == number->dp) {
		number->ip	= (char *)p;
	}
	else {
		if ((*p != ' ') && !isdigit(*p)) {
			return	NULL; /* Error: invalid arguments */
		}
		while ((p < number->dp) && *p == ' ') {	++p;	}
		while ((p < number->dp) && *p == '0') {	++p;	}
		number->ip	= (char *)p;
	}

	/* test: integer part is filled with digits? */
	if (number->ip != number->dp) {
		if (is_sequence_of_digits(number->ip, number->dp) == 0) {
			return	NULL;	/* Error: invalid arguments */
		}
	}

	/* test: numeral means zero? */
	if (is_zero(number) != 0) {
		number->sign	= Sign_e_positive; /* zero has positive sign */
	}

	return	number;	/* normal end */
}

/* compare two Number_t structures 'a' and 'b'
 * if `a' is less than `b', this returns -1.
 * if `a' equals `b', this returns zero.
 * if `a' is greater than `b', this returns 1.
 */
static int compare(Number_t	*a, Number_t	*b)
{
	size_t		a_len, b_len, limit;
	int			result;

	/* compare sign */
	if ((a->sign == Sign_e_negative) && b->sign == Sign_e_positive) {
			return	-1;	/* `a' is less than `b'		*/
	}
	else if ((b->sign == Sign_e_negative) && a->sign == Sign_e_positive) {
			return	1;	/* `a' is greater than `b'	*/
	}

	/* compare integer parts */
	a_len = (size_t)(a->dp - a->ip); b_len = (size_t)(b->dp - b->ip);
	if (a_len > b_len) {
		/* `a' is greater/less than `b' */
		return	((a->sign == Sign_e_positive)?	1:	-1);
	}
	else if (a_len < b_len) {
		/* `a' is less/greater than `b' */
		return	((a->sign == Sign_e_positive)?	-1:	1);
	}
	else if (a_len > 0) {
		if ((result = memcmp(a->ip, b->ip, a_len)) > 0) {
			/* `a' is greater/less than `b' */
			return	((a->sign == Sign_e_positive)?	1:	-1);
		}
		else if (result < 0) {
			/* `a' is less/greater than `b' */
			return	((a->sign == Sign_e_positive)?	-1:	1);
		}
	}

	/* compare fraction parts */
	a_len = (size_t)(a->ep - a->dp); b_len = (size_t)(b->ep - b->dp);
	limit	= ((a_len < b_len)? a_len: b_len);
	if (limit > 0) {
		if ((result = memcmp(a->dp , b->dp, limit)) > 0) {
			/* `a' is greater/less than `b' */
			return	((a->sign == Sign_e_positive)?	1:	-1);
		}
		else if (result < 0) {
			/* `a' is less/greater than `b' */
			return	((a->sign == Sign_e_positive)?	-1:	1);
		}
	}
	if (a_len > b_len) {
		/* `a' is greater/less than `b' */
		return	((a->sign == Sign_e_positive)?	1:	-1);
	}
	else if (a_len < b_len) {
		/* `a' is less/greater than `b' */
		return	((a->sign == Sign_e_positive)?	-1:	1);
	}

	return	0;	/* `a' equals `b' */
}

/* convert a digit code to integer */
static int digit2int(const char	*p)
{
	char	buf[2];

	buf[0] = *p; buf[1] = '\0'; return atoi(buf);
}

/* convert an integer to a digit code */
static char int2digit(int d)
{
	static char	digits[] =	{ '0', '1', '2', '3', '4'
							, '5', '6', '7', '8', '9'
							};
	return	digits[d];
}

/* add two Number_t structure `x' and `y'
 * signs of `x' and `y' must be `Sign_e_positive'
 */
/*@null@*/ static char *add(Number_t	*x
							, Number_t	*y
							, char		*result
							, size_t	result_size
							)
{
	const char	*xp, *yp;
	size_t		i, x_len, y_len, move_len;
	char		*rp;
	int			carry;

	/* set end of string */
	rp = result + (result_size - 1); *rp = '\0';

	/* process fraction parts */
	if ((x->ep != x->dp) || y->ep != y->dp) {
		if (x->ep == x->dp) {
			/* `x' has no fraction parts, `y' has it */
			move_len	= (size_t)(y->ep - y->dp)
			; rp		-= move_len
			; memcpy(rp, y->dp, move_len)
			; xp	= x->dp; yp = y->dp; carry = 0
			;
		}
		else if (y->ep == y->dp) {
			/* `y' has no fraction part, `x' has it */
			move_len	= (size_t)(x->ep - x->dp)
			; rp		-= move_len
			; memcpy(rp, x->dp, move_len)
			; xp	= x->dp; yp = y->dp; carry = 0
			;
		}
		else {
			/* adding the fraction digits */
			x_len = (size_t)(x->ep - x->dp)
			; y_len = (size_t)(y->ep - y->dp)
			;
			if (x_len > y_len) {
				/* `x' fraction part is longer than `y's */
				move_len	= x_len - y_len
				; rp		-= move_len
				; xp		= x->dp + (x_len - move_len)
				; yp		= y->dp + y_len
				; memcpy(rp, xp, move_len)
				; move_len	= y_len
				;
			}
			else if (x_len < y_len) {
				/* `y' fraction part is longer than `x's */
				move_len	= y_len - x_len
				; rp		-= move_len
				; xp		= x->dp + x_len
				; yp		= y->dp + (y_len - move_len)
				; memcpy(rp, yp, move_len)
				; move_len	= x_len
				;
			}
			else {
				/* 'x' fraction part is such length as `y's */
				xp			= x->dp + x_len
				; yp		= y->dp + y_len
				; move_len	= x_len
				;
			}

			/* add the digits... */
			for (i = 1, carry = 0; i < move_len; ++i) {
				--rp; --xp; --yp
				; carry	+= (digit2int(xp) + digit2int(yp))
				; *rp	= 	int2digit(carry % 10)
				; carry	/= 10
				;
			}

			/* set decimal point */
			--rp; --xp; --yp; *rp = *xp;
		}
	}
	else {
		/* `x' and `y' has no fraction part */
		xp = x->ep; yp = y->ep; carry = 0;
	}

	/* process integer parts */
	x_len = (size_t)(xp - x->ip); y_len = (size_t)(yp - y->ip);
	move_len	= ((x_len < y_len)?	x_len: y_len);
	for (i = 0; i < move_len; ++i) {
		--rp; --xp; --yp
		; carry	+= (digit2int(xp) + digit2int(yp))
		; *rp	= int2digit(carry % 10)
		; carry	/= 10
		;
	}

	/* process carry for integer part... */
	if (y_len > x_len) {
		/* 'y's integer part is longer than `x's */
		xp	= yp; /* process `y's integer part */
		move_len	= y_len - move_len;
	}
	else {
		/* 'x's integer part is longer than `y's */
		move_len	= x_len - move_len;
	}
	for (i = 0; i < move_len; ++i) {
		--rp; --xp; --yp
		; carry	+= digit2int(xp)
		; *rp	= int2digit(carry % 10)
		; carry	/= 10
		;
	}
	if (carry != 0) {
		--rp; *rp = int2digit(carry); carry = 0;
	}

	/* set zero ('0'), if integer part is empty */
	if ((*rp == '\0') || *rp == '.') {
		--rp; *rp = int2digit(0);
	}

	/* move the result string to the left */
	if (result < rp) {	memmove(result, rp, (strlen(rp) + 1));	}

	return	result; /* return the result buffer address */
}

/* subtract two Number_t structures `x' and `y'
 * signs of `x' and `y' must be Sign_e_positive
 * `x' should be greater than `y', or `x' should equals `y'
 */
/*@null@*/ static char *sub(Number_t	*x
							, Number_t	*y
							, char		*result
							, size_t	result_size
							)
{
	const char	*xp, *yp;
	size_t		x_len, y_len, len, i;
	char		*rp;
	int			bollow;

	rp = result + (result_size - 1); *rp = '\0';

	/* process the fraction part */
	if ((x->dp != x->ep) || y->dp != y->ep) {
		/* `x' or `y' has fraction parts */
		x_len	= (size_t)(x->ep - x->dp)
		; y_len	= (size_t)(y->ep - y->dp)
		;
		if (x_len > y_len) {
			/* `x's fraction part is longer than `y's */
			len	= x_len - y_len
			; xp = x->ep - len; x_len -= len; yp = y->ep
			; rp -= len
			; memcpy(rp, xp, len)
			; bollow = 0
			;
		}
		else if (x_len < y_len) {
			/* `y's fraction part is longer than `x's */
			len	= y_len - x_len; xp = x->ep
			; yp = y->ep; y_len -= len
			;
			if (x_len == 0) {
				/* we don't process the decimal point digit on `y' */
				--len;
			}
			for (i = 0, bollow = 10; i < len; bollow = 9, ++i) {
				--rp; *rp = int2digit(bollow - digit2int(--yp));
			}
			if (x_len == 0) {
				/* set the decimal point */
				--rp; *rp = *(--yp);
			}
			bollow	= 1;
		}
		else {
			/* `x' is such long as `y's */
			xp = x->ep; yp = y->ep; bollow = 0;
		}

		/* subtract fraction parts of `x' and 'y' */
		if (x_len > 0) {
			len = x_len - 1;
			for (i = 0; i < len; ++i) {
				--rp;
				if ((bollow = (digit2int(--xp)
								- bollow
								- digit2int(--yp)
								)) < 0) {
					*rp	= int2digit(10 + bollow);	bollow = 1;
				}
				else {
					*rp	= int2digit(bollow);		bollow = 0;
				}
			}
			--rp; *rp = *(--xp); --yp; /* set the decimal point */
		}
	}
	else {
		/* `x' and `y' has no fraction parts */
		xp = x->dp; yp = y->dp; bollow = 0;
	}

	/* process the integer parts */
	x_len = (size_t)(x->dp - x->ip); y_len = (size_t)(y->dp - y->ip);
	for (i = 0; i < y_len; ++i) {
		--rp;
		if ((bollow = digit2int(--xp) - bollow - digit2int(--yp)) < 0) {
			*rp	= int2digit(10 + bollow);	bollow = 1;
		}
		else {
			*rp	= int2digit(bollow);		bollow = 0;
		}
	}
	len	= x_len - y_len;
	for (i = 0; i < len; ++i) {
		--rp;
		if ((bollow = digit2int(--xp) - bollow) < 0) {
			*rp	= int2digit(10 + bollow);	bollow = 1;
		}
		else {
			*rp	= int2digit(bollow);		bollow = 0;
		}
	}

	/* truncate zero codes */
	while (((*rp != '\0') && *(rp + 1) != '.') && *rp == '0') {	++rp; }

	/* set zero ('0'), if integer part is empty */
	if ((*rp == '\0') || *rp == '.') {
		--rp; *rp = int2digit(0);
	}

	/* move the result string to the left */
	if (result < rp) {	memmove(result, rp, (strlen(rp) + 1));	}

	return	result;	/* return the result buffer address */
}

/* multiplies two Number_t structures `x' and `y'
 * signs of `x' and `y' must be Sign_e_positive
 */
/*@null@*/ static char *mul(Number_t	*x
							, Number_t	*y
							, char		*result
							, size_t	result_size
							)
{
	static int	muls[10][10] =
							{ {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 }
							, {  0,  1,  2,  3,  4,  5,  6,  7,  8,  9 }
							, {  0,  2,  4,  6,  8, 10, 12, 14, 16, 18 }
							, {  0,  3,  6,  9, 12, 15, 18, 21, 24, 27 }
							, {  0,  4,  8, 12, 16, 20, 24, 28, 32, 36 }
							, {  0,  5, 10, 15, 20, 25, 30, 35, 40, 45 }
							, {  0,  6, 12, 18, 24, 30, 36, 42, 48, 54 }
							, {  0,  7, 14, 21, 28, 35, 42, 49, 56, 63 }
							, {  0,  8, 16, 24, 32, 40, 48, 56, 64, 72 }
							, {  0,  9, 18, 27, 36, 45, 54, 63, 72, 81 }
							};
	struct segment_t {
		const char	*bp, *ep; /* begin and end pointer */
	}					xseg[2], yseg[2];
	const char			*xp, *yp;
	size_t				i, j;
	char				*rp, *sp, *rbp;
	int					carry, yval;

	/* initialize `result' buffer */
	rbp = rp = result + (result_size - 1); *rp = '\0';
	if ((result_size - 1) > 0) {
		memset(result, (int)'0', (result_size - 1));
	}

	/* initialize integer/fraction segments of `x' and `y' */
	xseg[0].bp = x->dp;			xseg[0].ep = x->ep - 1
	; xseg[1].bp = x->ip - 1;	xseg[1].ep = x->dp - 1
	; yseg[0].bp = y->dp;		yseg[0].ep = y->ep - 1
	; yseg[1].bp = y->ip - 1;	yseg[1].ep = y->dp - 1
	;

	/* process the numbers */
	for (i = 0; i < (sizeof(yseg) / sizeof(yseg[0])); ++i) {
		for (yp = yseg[i].ep; yp > yseg[i].bp; --yp, --rp) {
			sp = rp; yval = digit2int(yp); carry = 0;
			for (j = 0; j < (sizeof(xseg) / sizeof(xseg[0])); ++j) {
				for (xp = xseg[j].ep; xp > xseg[j].bp; --xp) {
					/* multiplies the digits */
					carry += (muls[yval][digit2int(xp)]
								+ digit2int(--sp)
								)
					; *sp = int2digit(carry % 10); carry /= 10
					;
				}
			}

			/* process carry */
			while (carry > 0) {
				--sp; *sp = int2digit(carry % 10); carry /= 10;
			}

			/* set most left digit of result */
			if (rbp > sp) {	rbp	= sp;	}
		}
	}

	/* construct the result string */
	/* get end pointer to result buffer (`rp') */
	rp	= result + (result_size - 1);
	/* get fraction length of result buffer (`i') */
	if ((i = (size_t)(x->ep - x->dp)) != 0) {	--i;		}
	if ((j = (size_t)(y->ep - y->dp)) != 0) {	i += (--j);	}
	/* get pointer to the fraction part (`sp') */
	sp	= rp - i;
	if (rbp < sp) {
		/* result has integer part */
		j	= (size_t)(sp - rbp); /* get integer part length */
		if (rbp != result) {
			memmove(result, rbp, j); /* move integer part */
		}
		if (i != 0) {
			/* result has fraction part */
			result[j]	= '.'; /* put decimal point */
			if ((result + (j + 1)) < sp) {
				/* move fraction part */
				memmove((result + (j + 1)), sp, (i + 1));
			}
		}
		else {
			/* result has no fraction part */
			result[j]	= '\0'; /* set end of string */
		}
	}
	else {
		/* result has no integer part */
		/* set leading string "0." to result buffer */
		memcpy(result, "0.", 2);
		/* move the fraction part (or end of string) */
		if ((result + 2) != sp) {
			memmove((result + 2), sp, i + 1);
		}
	}

	return	result; /* returns result buffer */
}

/* divide two `x' and `y'
 * signs of `x' and `y' must be Sign_e_positive
 */
/*@null@*/ static char *div_str(Result_t	*x
								, Result_t	*y
								, char		*result
								, size_t	fractions
								)
{
	size_t	i, remains_len;
	char	*ys[11], *numerals[2], *p, *xp, *rp, *remains = NULL;

	/* initialize working memories */
	for (i = 0; i < (sizeof(numerals) / sizeof(numerals[0])); ++i) {
			numerals[i]	= NULL;
	}
	for (i = 0; i < (sizeof(ys) / sizeof(ys[0])); ++i) { ys[i] = NULL; }

	/* set table of numerals of multiplied `y' */
	for (p = y->bufp; (*p != '\0') && *p == '0'; ++p) {	;	}
	numerals[0] = (char *)p;
	for (i = 0; i < (sizeof(ys) / sizeof(ys[0])); ++i) {
		char	w[3];

		(void)snprintf(w, sizeof(w), "%d", (int)i); numerals[1] = w;
		if ((ys[i] = aggregate_numbers(
							  Aggregate_e_opcode_mul
							, (const char **)numerals
							, (sizeof(numerals) / sizeof(numerals[0]))
							, 0
							)) == NULL) {
			/* Error: maybe, insufficient memory */
			result = NULL; goto DIV_ERROR;
		}
	}

	/* process the numeral strings... */
	/* set `result' and `remains' to the empty string */
	rp = result; *rp = '\0'; remains_len = 1;
	if ((remains = malloc(remains_len)) == NULL) {
		/* Error: insufficient memory */
		errno = ENOMEM; goto DIV_ERROR;
	}
	remains[0] = '\0'; numerals[0] = NULL;
	for (xp = x->bufp; *xp != '\0'; ++xp) {
		int	cmp;

		/* set sub-string of `x' to numerals[0] */
		remains_len	= strlen(remains);
		if ((p = malloc(remains_len + 2)) == NULL) {
			/* Error: insufficient memory */
			result = NULL; goto DIV_ERROR;
		}
		strcpy(p, remains)
		; p[remains_len]		= *xp
		; p[(remains_len + 1)]	= '\0'
		; numerals[0]			= p;
		; free(memset(remains, (int)'\0', remains_len)); remains = NULL
		; remains_len			+= 2
		;

		/* search the `ys[]' table, and set a digit of result */
		for (i = 0; i < (sizeof(ys) / sizeof(ys[0])); ++i) {
			if (compare_numbers(numerals[0], ys[i], &cmp) == NULL) {
				result = NULL; goto DIV_ERROR; /* Error: not expected */
			}
			if (cmp == 0) {	break;		}
			if (cmp < 0) {	--i; break;	}
		}
		*rp++ = int2digit((int)i); *rp = '\0';

		/* set remains as that `sub-string of x' minus `ys[i]' */
		numerals[1]	= ys[i];
		remains = aggregate_numbers(
							  Aggregate_e_opcode_sub
							, (const char **)numerals
							, (sizeof(numerals) / sizeof(numerals[0]))
							, 0
							);
		memset(numerals[0], (int)'\0', remains_len); free(numerals[0]);
		if (remains == NULL) {
			/* Error: maybe, insufficient memory */
			result = NULL; goto DIV_ERROR;
		}
	}

	/* format the result string */
	if (fractions > 0) {
		size_t	gap = (size_t)(((size_t)(rp - result) != fractions)? 1
								:									 2
								);

		for (i = 0, xp = rp, ++fractions; i < fractions; --xp, ++i) {
			*(xp + gap)	= *xp;
		}
		*(xp + gap--)	= '.';
		if (gap > 0) {	*(++xp)	= '0';	}
	}
	for (p = result; (*p != '\0') && *p == '0'; ++p) {	;	}
	if (p > result) {
		if ((*p == '\0') || *p == '.') {	--p;	}
		if (result < p) {	memmove(result, p, (strlen(p) + 1));	}
	}

DIV_ERROR:
	/* release the allocated memories */
	if (remains != NULL) {
		memset(remains, (int)'\0', strlen(remains))
		; free(remains); remains = NULL
		;
	}
	for (i = 0; i < (sizeof(ys) / sizeof(ys[0])); ++i) {
		if (ys[i] != NULL) {
			(void)destroy_aggregate_numbers(ys[i]); ys[i] = NULL;
		}
	}

	return	result; /* if several errors happened, return NULL */
}

/* allocate the result buffer (sums[k]) */
/*@null@*/ static Result_t *allocate_result_buffer(Result_t		*result
													, Number_t	*a
													, Number_t	*b
													)
{
	size_t	size = ((((a->dp - a->ip) > (b->dp - b->ip))?
									(a->dp - a->ip): (b->dp - b->ip))
						+ (((a->ep - a->dp) > (b->ep - b->dp))?
									(a->ep - a->dp): (b->ep - b->dp))
						+ (size_t)4
						);
	char	*bufp;

	if (result->size < size) {
		if ((bufp = malloc(size)) == NULL) {
			return	NULL; /* Error: insufficient memory */
		}

		/* replace the alocated buffer */
		if (result->bufp != NULL) {
			/* release allocated memory for the current buffer */
			free(memset(result->bufp, (int)'\0', result->size));
		}
		result->size = size; result->bufp = bufp;
	}

	return	result; /* address to the replaced buffer */
}

/*@null@*/ static char	*retrieve_the_first_numeral(const char	*numerals
													, Result_t	*sums
													)
{
	/* set the first numeral to `result' buffer */
	if ((sums->bufp = malloc((sums->size = (strlen(numerals) + 1))))
																== NULL
	) {	return NULL; /* Error: insufficient memories */	}

	return	strcpy(sums->bufp, numerals); /* return normally */

}

/* API: aggregate the numeral strings
 * this returns address of the allocated result buffer
 */
/*@null@*/ char *aggregate_numbers(Aggregate_e_opcode	opcode
									, const char		**numerals
									, size_t			n_numerals
									, ssize_t			scale
									)
{
	Result_t	sums[2], x, y;
	Number_t	a, b;
	size_t		i, j, k, l, result_left_size;
	char		*retval, *resultp;
	int			sv_errno = errno;

	/* test: arguments are invalid */
	if ((((numerals == NULL)
	|| n_numerals < 1)
	|| opcode < Aggregate_e_opcode_add)
	|| opcode >= Aggregate_e_opcode_n
	) {
		errno = EINVAL; return NULL; /* Error: invalid arguments */
	}

	/* initialize the result buffer */
	sums[1].size = (size_t)0; sums[1].bufp = NULL;

	/* initialize the parameter buffer of `div()' */
	x.size = y.size = (size_t)0; x.bufp = y.bufp = NULL;

	/* for unity (n_numerals is one, opcode is subtract or divide) */
	if ((n_numerals == 1)
	&& (opcode == Aggregate_e_opcode_sub
		|| opcode == Aggregate_e_opcode_div)
	) {
		Aggregate_e_opcode	op;
		const char			*numes[2];
		size_t				n_numes = 0;

		if (opcode == Aggregate_e_opcode_sub) {
			/* result means `-x' */
			op					= Aggregate_e_opcode_mul
			; numes[n_numes++]	= "-1"
			; numes[n_numes++]	= numerals[0]
			;
		}
		else {
			/* result means `1/x' */
			op					= Aggregate_e_opcode_div
			; numes[n_numes++]	= "1"
			; numes[n_numes++]	= numerals[0]
			;
		}
		if ((resultp = aggregate_numbers(op, numes, n_numes, scale))
																== NULL
		) {	return resultp; /* Error */	}

		retval	= retrieve_the_first_numeral(resultp, sums)
		; (void)destroy_aggregate_numbers(resultp)
		;
	}
	else { /* otherwise, result of unity is same as the first numerals */
		retval	= retrieve_the_first_numeral(numerals[0], sums);
	}

	/* aggregate numerals[1] to numerals[n_numerals]... */
	for (i = 1, j = 0; (retval != NULL) && i < n_numerals; ++i, j = k) {
		/* test: arguments are invalid? */
		if (((numerals[i] == NULL)
		|| make_number(sums[j].bufp,	&a) == NULL)
		|| make_number(numerals[i],		&b) == NULL
		) {
			/* Error: invalid arguments */
			errno = EINVAL;
CLEAN_UP:	/* release the allocated buffers of sums[] */
			for (l = 0; l < (sizeof(sums) / sizeof(sums[0])); ++l) {
				if (sums[l].bufp != NULL) {
					free(memset(sums[l].bufp, (int)'\0', sums[l].size))
					; sums[l].bufp = NULL;sums[l].size = (size_t)0
					;
				}
			}

			retval = NULL; break; /* Error: returns NULL */
		}

		/* set index of `sums[]' as result of calculating */
		k	= ((j + 1) % (sizeof(sums) / sizeof(sums[0])));

		if (opcode == Aggregate_e_opcode_mul) {
			/* allocate the result buffer (sums[k]) */
			result_left_size = (
						(((a.dp - a.ip) == 0)? 1: (a.dp - a.ip))
						+ (((b.dp - b.ip) == 0)? 1: (b.dp - b.ip))
						+ (((a.ep - a.dp) == 0)? 0: ((a.ep - a.dp) - 1))
						+ (((b.ep - b.dp) == 0)? 0: ((b.ep - b.dp) - 1))
						+ (size_t)3
						);
			if (sums[k].size < result_left_size) {
				if ((resultp = malloc(result_left_size)) == NULL) {
					/* Error: can't allocate the result buffer */
					errno = ENOMEM; goto CLEAN_UP;
				}
				if (sums[k].bufp != NULL) {
					free(memset(sums[k].bufp, (int)'\0', sums[k].size));
				}
				sums[k].size = result_left_size; sums[k].bufp = resultp;
			}

			if ((is_zero(&a) != 0) || is_zero(&b) != 0) {
				/* if `a' or `b' is zero, result is zero */
				retval	= strcpy(sums[k].bufp, "0");
			}
			else {
				/* set the sign of result, and address of digits */
				resultp	= sums[k].bufp;
				if (a.sign != b.sign) {
					*resultp = '-'; ++resultp
					; result_left_size	= sums[k].size - 1
					;
				}
				else {
					result_left_size	= sums[k].size;
				}

				retval	= mul(&a, &b, resultp, result_left_size);
			}
		}
		else if (opcode == Aggregate_e_opcode_div) {
			size_t	result_fraction_digits;

			/* is `b' zero? */
			if (is_zero(&b) != 0) {
				/* Error: divide by zero */
				errno = EDOM; goto CLEAN_UP;
			}

			/* set `x' that is shifted digits of `a' */
			result_fraction_digits	= (
							((a.ip == a.dp)? 0: (a.dp - a.ip))
							+ ((b.dp == b.ep)? 0: ((b.ep - b.dp) - 1))
							+ (size_t)1
							);
			if (scale >= (ssize_t)0) {
				result_fraction_digits += (size_t)scale;
				if (x.size < result_fraction_digits) {
					if ((resultp = malloc(result_fraction_digits))
																== NULL
					) {	errno = ENOMEM; goto CLEAN_UP;	}
					if (x.bufp != NULL) {
						memset(x.bufp, (int)'\0', x.size); free(x.bufp);
					}
					x.bufp = resultp; x.size = result_fraction_digits;
				}
				memset(x.bufp, (int)'0', (result_fraction_digits - 1));
				if (a.ip != a.dp) {
					memcpy(x.bufp, a.ip, (size_t)(a.dp - a.ip))
					; resultp	= x.bufp + (a.dp - a.ip)
					;
				}
				else {
					resultp	= x.bufp;
				}
				if (a.dp != a.ep) {
					memcpy(resultp, (a.dp + 1)
						, (((size_t)(a.ep - a.dp)
										< (x.size
											- (size_t)(resultp - x.bufp)
											)
							)?	(a.ep - a.dp - (size_t)1)
							:	(x.size - (resultp - x.bufp)
										- (size_t)1
										)
							)
						);
				}
				x.bufp[(result_fraction_digits - 1)]	= '\0'
				; result_fraction_digits	= (size_t)scale
				;
			}
			else {
				size_t	fraction;

				if (a.dp != a.ep) {
					if (b.dp == b.ep) {
						fraction	= (size_t)(a.ep - a.dp) - (size_t)1;
						result_fraction_digits += fraction;
					}
					else if ((a.ep - a.dp) > (b.ep - b.dp)) {
						fraction	= (size_t)(a.ep - a.dp)
									- (size_t)(b.ep - b.dp)
									;
						result_fraction_digits	+= fraction;
					}
					else {
						fraction	= 0;
					}
				}
				else {	fraction	= 0;	}
				if (x.size < result_fraction_digits) {
					if ((resultp = malloc(result_fraction_digits))
																== NULL
					) {	errno = ENOMEM; goto CLEAN_UP;	}
					if (x.bufp != NULL) {
						memset(x.bufp, (int)'\0', x.size); free(x.bufp);
					}
					x.bufp = resultp; x.size = result_fraction_digits;
				}
				memset(x.bufp, (int)'0', (result_fraction_digits - 1));
				if (a.ip != a.dp) {
					memcpy(x.bufp, a.ip, (size_t)(a.dp - a.ip))
					; resultp	= x.bufp + (size_t)(a.dp - a.ip)
					;
				}
				else {
					resultp	= x.bufp;
				}
				if (a.dp != a.ep) {
					memcpy(resultp
							, (a.dp + 1)
							, (a.ep - a.dp - (size_t)1)
							);
				}
				x.bufp[(result_fraction_digits - 1)]	= '\0'
				; result_fraction_digits				= fraction
				;
			}

			/* set `y' that is shifted digits of `b' */
			result_left_size = (((b.ip != b.dp)? (b.dp - b.ip) : 1)
								+ ((b.dp != b.ep)?	(b.ep - b.dp - 1)
									:				0
									)
								+ (size_t)1
								);
			if (y.size < result_left_size) {
				if ((resultp = malloc(result_left_size)) == NULL) {
					errno = ENOMEM; goto CLEAN_UP;
				}
				if (y.bufp != NULL) {
					memset(y.bufp, (int)'\0', y.size); free(y.bufp);
				}
				y.bufp		= resultp; y.bufp[0] = '\0'
				; y.size	= result_left_size
				;
			}
			if (b.ip != b.dp) {
				memcpy(y.bufp, b.ip, (size_t)(b.dp - b.ip))
				; resultp	= y.bufp + (b.dp - b.ip)
				;
			}
			else {
				resultp	= y.bufp;
			}
			if (b.dp != b.ep) {
				memcpy(resultp, (b.dp + 1), (size_t)(b.ep - b.dp - 1))
				; resultp	+= (b.ep - b.dp - 1)
				;
			}
			*resultp	= '\0';

			/* allocate the result buffer (sums[k]) */
			result_left_size	= (((a.ip != a.dp)? (a.dp - a.ip): 1)
								+ ((b.dp != b.ep)? (b.ep - b.dp - 1): 0)
								+ result_fraction_digits
								+ (size_t)3
								);
			if (sums[k].size < result_left_size) {
				if ((resultp = malloc(result_left_size)) == NULL) {
					/* Error: can't allocate the result buffer */
					errno = ENOMEM; goto CLEAN_UP;
				}
				if (sums[k].bufp != NULL) {
					free(memset(sums[k].bufp, (int)'\0', sums[k].size));
				}
				sums[k].size = result_left_size; sums[k].bufp = resultp;
			}

			/* set sign of result */
			resultp = sums[k].bufp;
			if (a.sign != b.sign) {
				*resultp++			= '-'
				; result_left_size	= sums[k].size - 1
				;
			}
			else {
				result_left_size	= sums[k].size;
			}

			/* 'x' is zero, or not? */
			if (is_zero(&a) != 0) {
				/* result of zero divided by anything is zero */
				resultp			= retval = sums[k].bufp
				; *resultp++	= '0'
				;
				if (result_fraction_digits != 0) {
					*resultp++	= '.'
					; memset(resultp, (int)'0', result_fraction_digits)
					; resultp	+= result_fraction_digits
					;
				}
				*resultp	= '\0';
			}
			else {
				/* calculate `x' divides by `y' */
				retval	= div_str(&x, &y
								, resultp, result_fraction_digits
								);
			}
		}
		else if (((opcode == Aggregate_e_opcode_sub)
					&& a.sign != b.sign
					)
		|| ((opcode == Aggregate_e_opcode_add) && a.sign == b.sign)
		) {
			Sign_e	sign;

			/* allocate the result buffer (sums[k]) */
			if (allocate_result_buffer((sums + k), &a, &b) == NULL) {
				/* Error: insufficient memory */
				errno = ENOMEM; goto CLEAN_UP;
			}

			/* set sign of result, and address of the digits */
			sign = a.sign; a.sign = b.sign = Sign_e_positive;
			if (sign == Sign_e_negative) {
				sums[k].bufp[0]			= '-'
				; resultp			= (sums[k].bufp + 1)
				; result_left_size = sums[k].size - 1
				;
			}
			else {
				resultp				= sums[k].bufp
				; result_left_size	= sums[k].size
				; 
			}

			retval	= add(&a, &b, resultp, result_left_size);
		}
		else if ((((opcode == Aggregate_e_opcode_sub)
					&& a.sign == Sign_e_positive)
					&& b.sign == Sign_e_positive)
		|| (((opcode == Aggregate_e_opcode_add)
					&& a.sign == Sign_e_positive)
					&& b.sign == Sign_e_negative)
		) {
			/* allocate the result buffer (sums[k]) */
			if (allocate_result_buffer((sums + k), &a, &b) == NULL) {
				/* Error: insufficient memory */
				errno = ENOMEM; goto CLEAN_UP;
			}

			a.sign = b.sign = Sign_e_positive;
			if (compare(&a, &b) < 0) {
				sums[k].bufp[0]	= '-'
				; retval		= sub(&b, &a
										, (sums[k].bufp + 1)
										, (sums[k].size - 1)
										);
			}
			else {
				retval	= sub(&a, &b, sums[k].bufp, sums[k].size);
			}
		}
		else {
			/* allocate the result buffer (sums[k]) */
			if (allocate_result_buffer((sums + k), &a, &b) == NULL) {
				/* Error: insufficient memory */
				errno = ENOMEM; goto CLEAN_UP;
			}

			a.sign = b.sign = Sign_e_positive;
			if (compare(&b, &a) < 0) {
				sums[k].bufp[0]	= '-'
				; retval		= sub(&a, &b
										, (sums[k].bufp + 1)
										, (sums[k].size - 1)
										);
			}
			else {
				retval	= sub(&b, &a, sums[k].bufp, sums[k].size);
			}
		}
	}

	/* if allocated buffer contains the last result,
	 * we must copy it to `result' buffer
	 */
	if ((retval != NULL) && j != 0) {
		Result_t	tmp_sum = sums[0];

		sums[0] = sums[1]; sums[1] = tmp_sum;
	}

	/* if the second slot of `sums[]' was allocated,
	 * release the allocated buffer
	 */
	if (sums[1].bufp != NULL) {
		memset(sums[1].bufp, (int)'\0', sums[1].size)
		; free((void *)(sums[1].bufp))
		; sums[1].bufp = NULL; sums[1].size = (size_t)0
		;
	}

	/* format the result numeral string */
	if ((retval != NULL) && opcode == Aggregate_e_opcode_div) {
		/* make minus-zero to be zero (when opcode means divide) */
		if (sums[0].bufp[0] == '-') {
			Number_t	zero;

			(void)make_number(sums[0].bufp, &zero);
			if (is_zero(&zero) != 0) {
				memmove(sums[0].bufp
						, (sums[0].bufp + 1)
						, strlen((sums[0].bufp + 1) + 1)
						);
			}
		}
	}
	else if ((retval != NULL) && scale >= (ssize_t)0) {
		/* set the result numeral string, according to scale */
		if ((resultp = strchr(sums[0].bufp, '.')) == NULL) {
			/* the result has no fraction part */
			result_left_size	= strlen(sums[0].bufp);
			if ((resultp = malloc((result_left_size + 1
													+ (size_t)scale
													+ 1
													))) == NULL
			) {	/* Error: insufficient memory */
INSUFFICIENT_RESULT_MEMORY:
				errno			= ENOMEM
				; free(memset(sums[0].bufp, (int)'\0', sums[0].size))
				; sums[0].bufp	= NULL; sums[0].size = (size_t)0
				;
				return	NULL; /* Error:  return NULL */
			}
			strcpy(resultp, sums[0].bufp);
			if (scale > 0) {
				resultp[result_left_size]	= '.'
				; memset((resultp + (result_left_size + 1))
						, (int)'0', (size_t)scale
						);
			}
		}
		else {
			/* the result has the fraction part */
			result_left_size	= (size_t)(resultp - sums[0].bufp);
			if ((resultp = malloc(result_left_size + 1 + scale + 1))
																== NULL
			) {	/* Error: insufficient memory */
				goto INSUFFICIENT_RESULT_MEMORY;
			}
			memcpy(resultp, sums[0].bufp, result_left_size);
			if ((i = strlen(sums[0].bufp + result_left_size))
												> ((size_t)scale + 1)
			) {
				memcpy((resultp + result_left_size)
						, (sums[0].bufp + result_left_size)
						, (size_t)(scale + 1)
						);
			}
			else {
				strcpy(resultp + result_left_size
						, sums[0].bufp + result_left_size
						);
				if (i != ((size_t)scale + 1)) {
					memset((resultp + (result_left_size + i))
							, (int)'0'
							, ((size_t)scale - i + 1)
							);
				}
			}
		}
		resultp[result_left_size + 1 + scale]	= '\0';

		/* release the buffer pointed from sums[0].bufp */
		free(memset(sums[0].bufp, (int)'\0', sums[0].size))
		; sums[0].bufp = resultp /* set the allocated result buffer */
		;
	}

	/* release the parameters of `Aggregate_e_op_div' */
	if (x.bufp != NULL) {
		free(memset(x.bufp, (int)'\0', x.size))
		; x.bufp = NULL; x.size = 0
		;
	}
	if (y.bufp != NULL) {
		free(memset(y.bufp, (int)'\0', y.size))
		; y.bufp = NULL; y.size = 0
		;
	}

	/* if several errors happened, this API returns NULL.
	 * otherwise, this returns address of `result' buffer
	 */
	return	((retval != NULL)?	((errno = sv_errno), sums[0].bufp)
			:					NULL
			);
}

static char *fill_zero_to_fraction_of_rounded(char	*rp, size_t	cnt)
{
	return	((char *)memset((rp - (cnt - (size_t)1)), (int)'0', cnt)
																- 1
			);
}

static int get_carry_of_rounded_digits(
									  Aggregate_e_round_how 	how
									, const char				*digit
									)
{
	int	val = digit2int(digit);

	return	 ((how == Aggregate_e_round_up
				|| (how == Aggregate_e_round_even && val > 4)
				)?	1	:	0
				);
}

static char *carry_up(char			*rp
					, const char	*np
					, const char	*stop
					, int			*carry
					)
{
	int	val;

	while (np >= stop) {
		val	= digit2int(np--) + *carry;
		*rp--	= int2digit(val % 10);
		*carry	= val / 10;
	}

	return	rp;
}

/* API: round the numeral string */
/*@null@*/ char *round_numbers(Aggregate_e_round_how	how
								, const char			*num
								, ssize_t				round_scale
								)
{
	const char	*np;
	Number_t	a, b;
	size_t		rounded_size;
	char		*rp, *rounded = NULL;
	int			carry, sv_errno = errno;

	/* test: arguments are invalid
	 * convert the numeral strings to Number_t structures
	 */
	if (((((how < Aggregate_e_round_even)
	|| how >= N_Aggregate_e_round_how)
	|| num == NULL)
	|| round_scale == (ssize_t)(MINIMUM_SSIZE_T))
	|| make_number(num, &a) == NULL
	) {	errno = EINVAL; return NULL; /* Error: invalid arguments */	}

	/* the numeral string is zero? */
	if (is_zero(&a) != 0) {
		/* the numerals string means zero.
		 * thus, the rounded string is zero, too
		 */
		if ((rounded = make_zero_numbers(round_scale - 1)) == NULL) {
			return NULL; /* Error: insufficient memory */
		}

		errno = sv_errno; return rounded; /* returns normally */
	}

	/* the precision of scale is lower than the numeral string? */
	if ((round_scale <= 0)
	&& (size_t)(-round_scale) > ((a.dp - a.ip) - (size_t)1)
	) {	/* the rounded string is zero, too */
		rounded_size	= 2;
		if ((rounded = malloc(rounded_size)) == NULL) {
			return NULL; /* Error: insufficient memory */
		}

		/* returns normally */
		errno = sv_errno; return strcpy(rounded, "0");
	}

	/* allocate memroy for the rounded numeral string */
	rounded_size	= (((a.sign == Sign_e_negative)? 1: 0)
						+ ((a.ip == a.dp)? 1: (a.dp - a.ip + 1))
						+ (size_t)(((round_scale < 0)
									|| round_scale == 1)?
										0	:	(round_scale + 1)
									)
						+ 1
						);
	if ((rounded = malloc(rounded_size)) == NULL) {
		return NULL; /* Error: insufficient memory */
	}
	rp = rounded + (rounded_size - 1); *rp-- = '\0';

	/* process the fraction part */
	if (round_scale > 0) {
		if (a.dp == a.ep) {
			/* fill zero fraction part of the rounded string */
			rp = fill_zero_to_fraction_of_rounded(
											  rp
											, ((size_t)round_scale - 1)
											)
			; carry = 0
			;
		}
		else {
			size_t	fraction_digits = ((a.ep - a.dp) - (size_t)1);

			if ((size_t)round_scale > fraction_digits) {
				/* precision of the scale is higer than the numeral */
				rp	= fill_zero_to_fraction_of_rounded(
							  rp
							, (size_t)(round_scale - fraction_digits -1)
							)
				; carry	= 0
				; np	= a.dp + fraction_digits;
				;
			}
			else {
				/* precision of the scale is equal or lower than
				 * the numeral
				 */
				carry	= get_carry_of_rounded_digits(
												  how
												, (a.dp + round_scale)
												)
				; np	= a.dp + (round_scale - 1)
				;
			}

			/* carry up the fraction part of the numeral string
			 * and set thee result to the rounded string
			 */
			rp	= carry_up(rp, np, (a.dp + 1), &carry);
		}

		/* if fraction part of the rounded string has digits,
		 * put the decimal point to the rounded string
		 */
		if (round_scale > 1) {	*rp-- = '.';	}
	}
	else {
		carry	= get_carry_of_rounded_digits(
										  how
										, ((a.ip == a.dp)?
											  "0"
											: (a.dp - (1 - round_scale))
											)
										);
	}

	/* process the integrer part */
	np	= a.dp - 1;
	if (round_scale > 0) {
		if (a.ip == a.dp) {
			*rp-- = int2digit(carry);
		}
		else {
			rp		= carry_up(rp, np, a.ip, &carry);
			if (carry > 0) {	*rp-- = int2digit(carry);	}
		}
	}
	else if (a.ip == a.dp) {
		round_scale	= -round_scale;
		if (round_scale > 0) {
			rp		= fill_zero_to_fraction_of_rounded(
												  rp
												, (size_t)round_scale
												);
		}
		*rp--	= int2digit(carry);
	}
	else {
		round_scale	= -round_scale; np -= round_scale;
		if (np < a.ip) {
			*rp--	= ((carry > 0)?	int2digit(carry): '0');
		}
		else {
			if (((how == Aggregate_e_round_down)
			&& np == a.ip) && carry == 0
			) {
				*rp-- = '0';
			}
			else {
				rp		= fill_zero_to_fraction_of_rounded(
												  rp
												, (size_t)round_scale
												)
				; rp	= carry_up(rp, np, a.ip, &carry)
				;
			}
		}
		if (carry > 0) {	*rp-- = int2digit(carry);	}
	}

	/* put sign to rounded string */
	(void)make_number(++rp, &b);
	if ((is_zero(&b) == 0) && a.sign == Sign_e_negative) {
		rounded[0] = '-'; np = rounded + 1;
	}
	else {
		np	= rounded;
	}

	/* move the integer and fraction part to the ahead of rounded */
	if (np < rp) {	memmove((char *)np, rp, (strlen(rp) + 1));	}

	/* returns address to another rounded string that is allocated */
	errno = sv_errno; return rounded;
}

/* API: duplicate the numeral string (allocate another new string) */
/*@null@*/ char *duplicate_numbers(const char	*num)
{
	Number_t	a;
	size_t		len;
	char		*duplicated = NULL;
	int			sv_errno = errno;

	/* test: arguments are invalid
	 * convert the numeral strings to Number_t structures
	 */
	if ((num == NULL)
	|| make_number(num, &a) == NULL
	) {	errno = EINVAL; return NULL; /* Error: invalid arguments */	}

	/* allocate another numeral string */
	if ((len = strlen(num)) > 0) {
		if ((duplicated = malloc(len + 1)) == NULL) {
			return	NULL; /* Error: insufficient memory */
		}
		memcpy(duplicated, num, len); duplicated[len] = '\0';
	}

	/* returns address to another allocated string */
	errno = sv_errno; return duplicated;
}

/* API: compare two numeral strings */
int *compare_numbers(const char	*a_str, const char	*b_str, int	*result)
{
	Number_t	a, b;
	int			sv_errno = errno;

	/* test: arguments are invalid
	 * convert the numeral strings to Number_t structures
	 */
	if (((result == NULL)
	|| make_number(a_str, &a) == NULL)
	|| make_number(b_str, &b) == NULL
	) {
		errno = EINVAL; return NULL; /* Error: invalid arguments */
	}

	/* return result of compare */
	*result	= compare(&a, &b); errno = sv_errno; return result;
}

/* API: predication: is the numeral string zero or not? */
int is_number_zero(const char	*num)
{
	Number_t	a;
	int			result, sv_errno = errno;

	/* test: arguments are invalid
	 * convert the numeral strings to Number_t structures
	 */
	if ((num == NULL) || make_number(num, &a) == NULL) {
		errno = EINVAL; return -1; /* Error: invalid arguments */
	}

	result	= is_zero(&a); /* predicate */

	/* returns 1, if the numeral string is zero.
	 * returns zero, it isn't zero
	 */
	errno = sv_errno; return result;
}

/* API: absolute the numeral string */
char	*absolute_numbers(const char	*num, ssize_t	scale)
{
	const char	*numerals[2];
	Number_t	a;
	size_t		n_numerals;
	char		*result;
	int			sv_errno = errno;

	/* test: invalid arguments */
	if ((num == NULL) || make_number(num, &a) == NULL) {
		errno = EINVAL; return NULL; /* Error: invalid arguments */
	}

	n_numerals					= 0
	; numerals[n_numerals++]	= ((a.sign == Sign_e_negative)?	"-1" : "1")
	; numerals[n_numerals++]	= num
	;
	if ((result = aggregate_numbers(Aggregate_e_opcode_mul
									, numerals
									, n_numerals
									, scale
									)) == NULL
	) {	return result; /* Error: insufficient memory */					}

	errno = sv_errno; return result; /* return normally */
}

/* API: make zero */
char *make_zero_numbers(ssize_t	scale)
{
	char	*result = malloc((size_t)((scale > 0)? (scale + 3): 2));
	int		sv_errno = errno;

	if (result != NULL) {
		result[0]		= '0';
		if (scale > 0) {
			result[1]				= '.'
			; memset(result + 2, (int)'0', (size_t)scale)
			; result[(scale + 2)]	= '\0'
			;
		}
		else {
			result[1]	= '\0';
		}
	}

	/* return NULL (error) or address of zero string */
	if (result != NULL) {	errno = sv_errno;	}
	return	result;
}

/* API: make power of ten */
char *make_power_of_ten(ssize_t	scale)
{
	char	*result;
	int		sv_errno = errno;

	if (scale > 0) {
		/* result is 10^(negative integer) */
		if ((result = malloc((size_t)(scale + 3))) != NULL) {
			result[0]	= '0'; result[1]	= '.';
			if (scale > 1) {
				memset((result + 2), (int)'0', (size_t)(scale - 1));
			}
			result[(scale + 1)]	= '1'; result[(scale + 2)]	= '\0';
		}
	}
	else {
		/* result is 10^(zero or positive integer) */
		scale	= -scale;
		if ((result = malloc((size_t)(scale + 2))) != NULL) {
			result[0]	= '1';
			if (scale > 0) {
				memset((result + 1), (int)'0', (size_t)scale);
			}
			result[(scale + 1)]	= '\0';
		}
	}

	/* if several errors occurred, returns NULL */
	if (result != NULL) {	errno = sv_errno;	}
	return	result;
}

/* API: shift the numeral string */
char *shift_numbers(const char	*num, ssize_t	scale)
{
	const char	*numerals[2];
	size_t		n_numerals;
	char		*result, *a;
	int			sv_errno = errno;

	if ((a = make_power_of_ten(scale)) == NULL) {
		return	NULL; /* Error: insufficient memory */
	}
	n_numerals	= 0
	; numerals[n_numerals++]	= num
	; numerals[n_numerals++]	= a
	; result = aggregate_numbers(Aggregate_e_opcode_mul
								, numerals
								, n_numerals
								, -1
								)
	; (void)destroy_aggregate_numbers(a); a	= NULL
	;

	/* if several error ovvurred, return NULL */
	if (result != NULL) {	errno = sv_errno;	}
	return	result;
}


/* subroutine for `sqrt_numbers()': search `i' for `block' */
static int search_i_for_block(const char	*block)
{
	static char	*squares[] =	{  "0",  "1",  "4",  "9", "16"
								, "25", "36", "49", "64", "81"
								, "100"
								};
	int			i, cmp_result;

	/* search `i' as the first value of `result'... */
	for (i = 0; i < (int)(sizeof(squares) / sizeof(squares[0])); ++i) {
		if (compare_numbers(squares[i], block, &cmp_result) == NULL)
		{	return	-1; /* FATAL: (never!) */	}
		else if (cmp_result > 0) {
			--i; break; /* found the first value of `result' */
		}
		else if (cmp_result == 0) {
			break; /* found the first value of `result' */
		}
		else { ; /* check the next... */	}
	}
	/* `i' never becomes `(sizeof(squares) / sizeof(squares[0]))' */

	return	i; /* returns `i' is matched `block' */
}

/* API: extraction square root	*/
char *sqrt_numbers(const char	*num, ssize_t	scale)
{
#	define		Throw_Sqrt_error(ERRNO) {					\
						errno = (ERRNO); goto Sqrt_error;	\
					}
	const char	*numerals[2], *ap;
	Number_t	a;
	size_t		n_numerals, result_size, block_len;
	char		block[(2 + 1)], i_str[(1 + 1)]
				, *resp, *stopp, *result = NULL
				, *u = NULL, *v = NULL, *w = NULL
				, *x = NULL, *y = NULL, *z = NULL
				;
	int			cmp_result, sv_errno = errno;

	/* test: invalid arguments */
	if (((num == NULL) || make_number(num, &a) == NULL)
	|| a.sign == Sign_e_negative
	) {
		Throw_Sqrt_error(EINVAL); /* Error: invalid arguments */

Sqrt_error:	{ /* occurred several errors */
			if (result != NULL) {
				(void)destroy_aggregate_numbers(result);
			}
			if (u != NULL) {	(void)destroy_aggregate_numbers(u);	}
			if (v != NULL) {	(void)destroy_aggregate_numbers(v);	}
			if (w != NULL) {	(void)destroy_aggregate_numbers(w);	}
			if (x != NULL) {	(void)destroy_aggregate_numbers(x);	}
			if (y != NULL) {	(void)destroy_aggregate_numbers(y);	}
			if (z != NULL) {	(void)destroy_aggregate_numbers(z);	}
			return	NULL; /* returns error */
		}
	}

	/* allocate the result buffer */
	result_size	= (size_t)(
						((a.ip != a.dp)? (((a.dp - a.ip) + 1) / 2):	1)
						+ ((scale > 0)?	 (scale + 2):				1)
						);
	if ((result = malloc(result_size)) == NULL) {
		Throw_Sqrt_error(errno); /* Error: insufficient memory */
	}
	result[0] = '\0'; resp = result; /* make result empty */

	/* initialize `x', `y' and `result' */
	if (a.ip == a.dp) {
		/* integer part of `num'is zero */
		ap = (a.dp + 1); /* set the next block source pointer */

		/* initialize `result' */
		*resp++ = '0';
		if (scale > 0) {	*resp++ = '.'; /*set decimal point */	}
		*resp = '\0';

		/* initialize `x' and `y' */
		if (((x = make_zero_numbers(0)) == NULL)
		|| (y = make_zero_numbers(0)) == NULL
		) {	Throw_Sqrt_error(errno); /* Error: insufficient memory */	}
	}
	else {
		/* process the first block */
		/* retrieve the first block of the number */
		block_len = (size_t)(
						(((a.dp - a.ip) % (sizeof(block) - 1)) != 0)?
												1: (sizeof(block) - 1)
						)
		; memcpy(block, a.ip, block_len)
		; block[block_len]	= '\0'
		;

		/* set the next block pointer */
		if ((ap = (a.ip + block_len)) == a.dp) {	++ap;	}

		/* search `i' as the first value of `result,
		 * and convert `i' to string
		 */
		i_str[0] = int2digit(search_i_for_block(block))
		; i_str[1] = '\0'
		;
																		

		/* initialize `result' := i */
		*resp++ = i_str[0];
		if ((a.dp < ap) && scale > 0) {
			*resp++	= '.'; /* set decimal point */
		}
		*resp = '\0';

		/* `u' := `i_str' + `i_str', `x' = `u' * 10 */
		n_numerals	= 0
		; numerals[n_numerals++]	= i_str
		; numerals[n_numerals++]	= i_str
		;
		if ((u = aggregate_numbers(Aggregate_e_opcode_add
									, numerals
									, n_numerals
									, -1
									)) == NULL
		) {	Throw_Sqrt_error(errno); /* Error: insufficient memory */	}
		n_numerals	= 0
		; numerals[n_numerals++]	= u
		; numerals[n_numerals++]	= "10"
		;
		if ((x = aggregate_numbers(Aggregate_e_opcode_mul
									, numerals
									, n_numerals
									, -1
									)) == NULL
		) {	Throw_Sqrt_error(errno); /* Error: insufficient memory */	}
		(void)destroy_aggregate_numbers(u); u = NULL
		;

		/* `u' := `i_str' * `i_str', 'y' := `block' - `u' */
		n_numerals	= 0
		; numerals[n_numerals++]	= i_str
		; numerals[n_numerals++]	= i_str
		;
		if ((u = aggregate_numbers(Aggregate_e_opcode_mul
									, numerals
									, n_numerals
									, -1
									)) == NULL
		) {	Throw_Sqrt_error(errno); /* Error: insufficient memory */	}
		n_numerals	= 0
		; numerals[n_numerals++]	= block
		; numerals[n_numerals++]	= u
		;
		if ((y = aggregate_numbers(Aggregate_e_opcode_sub
									, numerals
									, n_numerals
									, -1
									)) == NULL
		) {	Throw_Sqrt_error(errno); /* Error: insufficient memory */	}
		(void)destroy_aggregate_numbers(u); u = NULL
		;
	}

	/* process the each blocks (two columns of `num')... */
	for (stopp = result + (result_size - 1); resp < stopp; ) {
		/* retrieve the block */
		if (a.ep <= ap) {
			block[0] = '0'; block[1] = '\0';
		}
		else if ((ap < a.dp) || (a.ep - ap) > 1) {
			memcpy(block, ap, (sizeof(block) - 1))
			; block[(sizeof(block) - 1)]	= '\0'
			; ap	+= (sizeof(block) - 1)
			;
		}
		else {
			block[0] = *ap; block[1] = '0'; block[2] = '\0'; ++ap;
		}

		/* `y' := (100 * `y') + 'block' */
		n_numerals	= 0
		; numerals[n_numerals++]	= "100"
		; numerals[n_numerals++]	= y
		;
		if ((u = aggregate_numbers(Aggregate_e_opcode_mul
									, numerals
									, n_numerals
									, -1
									)) == NULL
		) {	Throw_Sqrt_error(errno); /* Error: insufficient memory */	}
		n_numerals	= 0
		; numerals[n_numerals++]	= u
		; numerals[n_numerals++]	= block
		;
		if ((v = aggregate_numbers(Aggregate_e_opcode_add
									, numerals
									, n_numerals
									, -1
									)) == NULL
		) {	Throw_Sqrt_error(errno); /* Error: insufficient memory */	}
		(void)destroy_aggregate_numbers(u);		u = NULL
		; (void)destroy_aggregate_numbers(y);	y = NULL
		; y	= v; v	= NULL
		;

		/* calculate `u', `v', `w'... */
		/* u := `y' / `x' (scale = 0) */
		if (is_number_zero(x) != 0) {
			/* if `x' is zero, we cannot use 'x' for setting `u' */
			/* search `i' that is matched `y',
			 * and convert `i' to string
			 */
			i_str[0] = int2digit(search_i_for_block(y))
			; i_str[1] = '\0'
			;

			/* set `u' */
			if ((u = duplicate_numbers(i_str)) == NULL) {
				/* Error: insufficient memory */
				Throw_Sqrt_error(errno);
			}
		}
		else {
			n_numerals	= 0
			; numerals[n_numerals++]	= y
			; numerals[n_numerals++]	= x
			;
			if ((u = aggregate_numbers(Aggregate_e_opcode_div
										, numerals
										, n_numerals
										, 0
										)) == NULL
			) {	/* Error: insufficient memory */
				Throw_Sqrt_error(errno);
			}
		}
		for (;;) {
			/* `w' := `x' + `u' */
			n_numerals	= 0
			; numerals[n_numerals++]	= x
			; numerals[n_numerals++]	= u
			;
			if ((w = aggregate_numbers(Aggregate_e_opcode_add
										, numerals
										, n_numerals
										, -1
										)) == NULL
			) {	/* Error: insufficient memory */
				Throw_Sqrt_error(errno);
			}

			/* `v' := `w' * `u' */
			n_numerals	= 0
			; numerals[n_numerals++]	= w
			; numerals[n_numerals++]	= u
			;
			if ((v = aggregate_numbers(Aggregate_e_opcode_mul
										, numerals
										, n_numerals
										, -1
										)) == NULL
			) {	/* Error: insufficient memory */
				Throw_Sqrt_error(errno);
			}

			/* compare `y' to `v': if (`y' >= `v'), we found the `u' */
			if (compare_numbers(y, v, &cmp_result) == NULL) {
				Throw_Sqrt_error(EFAULT); /* Error: (never!) */
			}
			if (cmp_result >= 0) {	break;	}

			/* increment `u' (`u' := `u' - 1) */
			n_numerals	= 0
			; numerals[n_numerals++]	= u
			; numerals[n_numerals++]	= "1"
			;
			if ((z = aggregate_numbers(Aggregate_e_opcode_sub
										, numerals
										, n_numerals
										, -1
										)) == NULL
			) {	/* Error: insufficient memory */
				Throw_Sqrt_error(errno);
			}
			(void)destroy_aggregate_numbers(u); u = NULL
			; u	= z; z	= NULL
			;

			/* release `w', `v' */
			(void)destroy_aggregate_numbers(w); w = NULL
			; (void)destroy_aggregate_numbers(v); v = NULL
			;
		}

		/* set result (use `u') */
		*resp++ = *u;
		if ((ap == a.dp) && scale > 0) {	*resp++ = '.'; ++ap;	}
		*resp	= '\0';

		/* `x' := ('w' + 'u') * 10 */
		n_numerals	= 0
		; numerals[n_numerals++]	= w
		; numerals[n_numerals++]	= u
		;
		if ((z = aggregate_numbers(Aggregate_e_opcode_add
									, numerals
									, n_numerals
									, -1
									)) == NULL
		) {	/* Error: insufficient memory */
			Throw_Sqrt_error(errno);
		}
		(void)destroy_aggregate_numbers(u); u = NULL;
		n_numerals	= 0
		; numerals[n_numerals++]	= z
		; numerals[n_numerals++]	= "10"
		;
		if ((u = aggregate_numbers(Aggregate_e_opcode_mul
									, numerals
									, n_numerals
									, -1
									)) == NULL
		) {	/* Error: insufficient memory */
			Throw_Sqrt_error(errno);
		}
		(void)destroy_aggregate_numbers(z);		z = NULL
		; (void)destroy_aggregate_numbers(x);	x = NULL
		; x	= u; u	= NULL
		;

		/* 'y' = 'y' - 'v' */
		n_numerals	= 0
		; numerals[n_numerals++]	= y
		; numerals[n_numerals++]	= v
		;
		if ((z = aggregate_numbers(Aggregate_e_opcode_sub
									, numerals
									, n_numerals
									, -1
									)) == NULL
		) {	/* Error: insufficient memory */
			Throw_Sqrt_error(errno);
		}
		(void)destroy_aggregate_numbers(y); y = NULL
		; y	= z; z	= NULL
		;

		/* release `w', `v' */
		(void)destroy_aggregate_numbers(w); w = NULL
		; (void)destroy_aggregate_numbers(v); v = NULL
		;
	}

	/* release the allocated working memories */
	if (x != NULL) {	(void)destroy_aggregate_numbers(x);	}
	if (y != NULL) {	(void)destroy_aggregate_numbers(y);	}

	errno = sv_errno; return result; /* return normally */
#	undef Throw_Sqrt_error
}

/* API: exponent of real numbers */
char *exponent_numbers(const char	*num, ssize_t	scale)
{
#	define		Throw_Exponent_error(ERRNO) {					\
						errno = (ERRNO); goto Exponent_error;	\
					}
	const char	*numerals[2];
	Number_t	x;
	ssize_t		div_scale;
	size_t		n_numerals, i, j;
	char		*exponents[2], *factorial_n = NULL, *power_of_num = NULL
				, *a = NULL, *n = NULL
				;
	int			the_number_is_negative = 0, sv_errno = errno;

	/* initialize exponents[2] */
	exponents[0] = exponents[1]	= NULL;

	/* test: arguments is valid? */
	if ((num == NULL) || make_number(num, &x) == NULL) {
		Throw_Exponent_error(EINVAL); /* Error: invalid arguments */

Exponent_error:	{ /* occurred several errors */
			if ((the_number_is_negative != 0) && num != NULL) {
				/* release `num' */
				(void)destroy_aggregate_numbers((char *)num); num = NULL;
			}
			if (factorial_n != NULL) {
				(void)destroy_aggregate_numbers(factorial_n);
			}
			if (power_of_num != NULL) {
				(void)destroy_aggregate_numbers(power_of_num);
			}
			if (a != NULL) {	(void)destroy_aggregate_numbers(a);	}
			if (n != NULL) {	(void)destroy_aggregate_numbers(n);	}
			for (i = 0
			; i < (sizeof(exponents) / sizeof(exponents[0]))
			; ++i
			) {
				if (exponents[i] != NULL) {
					(void)destroy_aggregate_numbers(exponents[i]);
				}
			}
			return	NULL; /* returns error */
		}
	}

	/* if the number is zero, result is one. */
	if (is_zero(&x) != 0) {
		i	= ((scale > 0)?	((size_t)scale + 3): 2);
		if ((exponents[0] = malloc(i)) == NULL) {
			Throw_Exponent_error(errno); /* Error: insufficient number */
		}
		*(exponents[0])	= '1';
		if (scale > 0) {
			*(exponents[0] + 1)	= '.'
			; memset((exponents[0] + 2), (int)'0', (size_t)scale)
			; *(exponents[0] + (scale + 2))		= '\0'
			;
		}
		else {
			*(exponents[0] + 1)	= '\0';
		}
		return	exponents[0]; /* return normally */
	}

	/* if the number is negative, set the sign to positive */
	the_number_is_negative	= ((x.sign == Sign_e_negative)? !0: 0);
	if (the_number_is_negative != 0) {
		x.sign = Sign_e_positive;

		/* `num' := -1 * `num' */
		n_numerals	= 0
		; numerals[n_numerals++]	= "-1"
		; numerals[n_numerals++]	= num
		;
		if ((a = aggregate_numbers(Aggregate_e_opcode_mul
									, numerals
									, n_numerals
									, -1
									)) == NULL
		) {	/* Error: insufficient memory */
			Throw_Exponent_error(errno);
		}
		; num	= a; a	= NULL
		;
	}

	/* initialize `exponents[0]', `n', `factorial_n', `power_of_num' */
	i	= 0; j	= 1;
	if (((((exponents[i] = duplicate_numbers("1")) == NULL)
	|| (n = duplicate_numbers("1")) == NULL)
	|| (factorial_n = duplicate_numbers("1")) == NULL)
	|| (power_of_num = duplicate_numbers(num)) == NULL
	) {
		Throw_Exponent_error(errno); /* Error: insufficient memory */
	}

	/* exp(num) := sigma{ (`power_of_num' ^ n) / n! ; 0 <= n } */
	for (div_scale = ((scale > 0)? (scale + 2): 2); ; i = j) {
		j	= (i + 1) %2; /* set index of result for this iteration */

		/* `a' := `power_of_num' / `factorial_n',
		 * `exponents[j]' := `exponents[i]' + `a'
		 */
		n_numerals	= 0
		; numerals[n_numerals++]	= power_of_num
		; numerals[n_numerals++]	= factorial_n
		;
		if ((a = aggregate_numbers(Aggregate_e_opcode_div
								, numerals
								, n_numerals
								, div_scale
								)) == NULL
		) {	/* Error: insufficient memory */
			Throw_Exponent_error(errno);
		}
		if (exponents[j] != NULL) {
			(void)destroy_aggregate_numbers(exponents[j])
			; exponents[j] = NULL
			;
		}
		n_numerals	= 0
		; numerals[n_numerals++]	= exponents[i]
		; numerals[n_numerals++]	= a
		;
		if ((exponents[j] = aggregate_numbers(Aggregate_e_opcode_add
											, numerals
											, n_numerals
											, div_scale
											)) == NULL
		) {	/* Error: insufficient memory */
			Throw_Exponent_error(errno);
		}
		if (make_number(a, &x) == NULL) {
			Throw_Exponent_error(errno); /* FATAL: (never!) */
		}
		(void)destroy_aggregate_numbers(a); a	= NULL;
		if (is_zero(&x) != 0) {
			break; /* we got the result */
		}

		/* `n' = `n' + 1 */
		n_numerals	= 0
		; numerals[n_numerals++]	= n
		; numerals[n_numerals++]	= "1"
		;
		if ((a = aggregate_numbers(Aggregate_e_opcode_add
									, numerals
									, n_numerals
									, -1
									)) == NULL
		) {	/* Error: insufficient memory */
			Throw_Exponent_error(errno);
		}
		(void)destroy_aggregate_numbers(n)
		; n	= a; a	= NULL
		;

		/* `factorial_n' := `factorial_n' * `n' */
		n_numerals	= 0
		; numerals[n_numerals++]	= factorial_n
		; numerals[n_numerals++]	= n
		;
		if ((a = aggregate_numbers(Aggregate_e_opcode_mul
											, numerals
											, n_numerals
											, -1
											)) == NULL
		) {	/* Error: insufficient memory */
			Throw_Exponent_error(errno);
		}
		(void)destroy_aggregate_numbers(factorial_n)
		; factorial_n	= a; a	= NULL
		;

		/* `power_of_num' := `power_of_num' * `num' */
		n_numerals	= 0
		; numerals[n_numerals++]	= power_of_num
		; numerals[n_numerals++]	= num
		;
		if ((a = aggregate_numbers(Aggregate_e_opcode_mul
											, numerals
											, n_numerals
											, -1
											)) == NULL
		) {	/* Error: insufficient memory */
			Throw_Exponent_error(errno);
		}
		(void)destroy_aggregate_numbers(power_of_num)
		; power_of_num	= a; a	= NULL
		;
	}

	/* release the working memories */
	if (a != NULL) {	(void)destroy_aggregate_numbers(a);	}
	if (n != NULL) {	(void)destroy_aggregate_numbers(n);	}
	if (factorial_n != NULL) {
		(void)destroy_aggregate_numbers(factorial_n)
		; factorial_n = NULL
		;
	}
	if (power_of_num != NULL) {
		(void)destroy_aggregate_numbers(power_of_num)
		; power_of_num  = NULL
		;
	}
	if (exponents[j] != NULL) {
		(void)destroy_aggregate_numbers(exponents[j])
		; exponents[j] = NULL
		;
	}

	/* the number is negative? */
	if (the_number_is_negative != 0) {
		/* release `num' */
		(void)destroy_aggregate_numbers((char *)num); num = NULL;

		/* `exponents[i]' := 1 / `exponents[i] */
		n_numerals	= 0
		; numerals[n_numerals++]	= "1"
		; numerals[n_numerals++]	= exponents[i]
		;
		if ((a = aggregate_numbers(Aggregate_e_opcode_div
									, numerals
									, n_numerals
									, div_scale
									)) == NULL
		) {	/* Error: insufficient memory */
			Throw_Exponent_error(errno);
		}
		(void)destroy_aggregate_numbers(exponents[i])
		; exponents[i]	= a; a	= NULL
		;
	}

	/* make a result according to `scale' */
	if ((a = round_numbers(Aggregate_e_round_down
							, exponents[i]
							, ((scale > 0)? (scale + 1): 1)
							)) == NULL
	) {	/* Error: insufficient memory */
		Throw_Exponent_error(errno);
	}
	(void)destroy_aggregate_numbers(exponents[i])
	; exponents[i]	= a ; a	= NULL
	;

	errno = sv_errno; return exponents[i]; /* return normally */
#	undef		Throw_Exponent_error
}

/* subroutine for `ln_numbers()' : look up table of ln(10) */
static const char	ln_of_10tbl[] =
"2.302585092994045684017991454684364207601101488628772976033327900967"\
"57260967735248023599720508959829834196778404228624863340952546508280"\
"67566662873690987816894829072083255546808437998948262331985283935053"\
"08965377732628846163366222287698219886746543667474404243274365155048"\
"93431493939147961940440022210510171417480036880840126470806855677432"\
"16228355220114804663715659121373450747856947683463616792101806445070"\
"64800027750268491674655058685693567342067058113642922455440575892572"\
"42082413146956890167589402567763113569192920333765871416602301057030"\
"89634572075440370847469940168269282808481184289314848524948644871927"\
"80967627127577539702766860595249671667418348570442250719796500471495"\
"10504922147765676369386629769795221107182645497347726624257094293225"\
"82798502585509785265383207606726317164309505995087807523710333101197"\
"85754733154142180842754386359177811705430982748238504564801909561029"\
"92918243182375253577097505395651876975103749708886921802051893395072"\
"38539205144634197265287286965110862571492198849978";
#define MAX_SCALE_OF_ln_of_10tbl (sizeof(ln_of_10tbl) - 3)
/*@null@*/ static char *lookup_ln_of_10tbl(ssize_t	scale)
{
	size_t	size = (((size_t)scale > MAX_SCALE_OF_ln_of_10tbl)?	
												sizeof(ln_of_10tbl)
					: 							(size_t)scale + 3
					);
	char	*result;

	if ((result = malloc(size)) == NULL) {
		return	NULL; /* Error: insufficient memory */
	}
	memcpy(result, ln_of_10tbl, (size - 1)); result[(size - 1)]	= '\0';

	return	result; /* return normally */
}

/* subroutine for `ln_numbers()' :
 *	Napierian-logarithm(x) (transformed Maclaurin series) is;
 * 		2 * sigma{	[i > 1],
 *					(((x - 1) / (x + 1)) ^ (2 * i - 1)) / (2 * i - 1)
 *					}
 */
/*@null@*/ static char *napierian_logarithm(const char	*num
										, ssize_t		scale
										)
{
#	define		Throw_Nlog_error(ERRNO)	{ lineno = __LINE__	\
										; errno = (ERRNO)	\
										; goto Nlog_error	\
										;					\
										}
	const char	*numerals[2];
	Number_t	x;
	ssize_t		div_scale;
	size_t		n_numerals, i, j;
	char		*nlogs[2], *ratio_of_num = NULL, *power_of_ratio = NULL
				, *shift_up_num = NULL, *a = NULL, *n = NULL
				;
	int			lineno = __LINE__;

	/* initialize nlogs[2] */
	nlogs[0] = nlogs[1]	= NULL;

	/* test: arguments is valid? */
	if ((num == NULL) || make_number(num, &x) == NULL) {
		Throw_Nlog_error(EINVAL); /* Error: invalid arguments */

Nlog_error:	{ /* occurred several errors */
			if (shift_up_num != NULL) {
				(void)destroy_aggregate_numbers(shift_up_num);
				if (num != NULL) {
					(void)destroy_aggregate_numbers((char *)num);
				}
			}
			if (ratio_of_num != NULL) {
				(void)destroy_aggregate_numbers(ratio_of_num);
			}
			if (power_of_ratio != NULL) {
				(void)destroy_aggregate_numbers(power_of_ratio);
			}
			if (n != NULL) {	(void)destroy_aggregate_numbers(n);	}
			if (a != NULL) {	(void)destroy_aggregate_numbers(a);	}
			for (i = 0; i < (sizeof(nlogs) / sizeof(nlogs[0])); ++i) {
				if (nlogs[i] != NULL) {
					(void)destroy_aggregate_numbers(nlogs[i]);
				}
			}
			return	NULL; /* returns error */
		}
	}

	/* set `div_scale' */
	div_scale	= ((scale > 0)? (scale * 2): 2);

	/* `nlogs[0]' := `power_of_ratio' := (`num' - 1) / (`num' + 1) */
	i	= 0
	; n_numerals	= 0
	; numerals[n_numerals++]	= num
	; numerals[n_numerals++]	= "1"
	;
	if ((a = aggregate_numbers(Aggregate_e_opcode_sub
								, numerals
								, n_numerals
								, -1
								)) == NULL
	) {	Throw_Nlog_error(errno); /* Error: insufficient memory */	}
	n_numerals	= 0
	; numerals[n_numerals++]	= num
	; numerals[n_numerals++]	= "1"
	;
	if ((n = aggregate_numbers(Aggregate_e_opcode_add
								, numerals
								, n_numerals
								, -1
								)) == NULL
	) {	Throw_Nlog_error(errno); /* Error: insufficient memory */	}
	n_numerals	= 0
	; numerals[n_numerals++]	= a
	; numerals[n_numerals++]	= n
	;
	if ((power_of_ratio = aggregate_numbers(Aggregate_e_opcode_div
								, numerals
								, n_numerals
								, div_scale
								)) == NULL
	) {	Throw_Nlog_error(errno); /* Error: insufficient memory */	}
	(void)destroy_aggregate_numbers(a);		a	= NULL
	; (void)destroy_aggregate_numbers(n);	n	= NULL
	;
	if ((nlogs[i] = duplicate_numbers(power_of_ratio)) == NULL) {
		Throw_Nlog_error(errno); /* Error: insufficient memory */
	}

	/* `ratio_of_num' := `exponents[0]' * `exponents[0]' */
	n_numerals	= 0
	; numerals[n_numerals++]	= nlogs[i]
	; numerals[n_numerals++]	= nlogs[i]
	;
	if ((ratio_of_num = aggregate_numbers(Aggregate_e_opcode_mul
								, numerals
								, n_numerals
								, div_scale
								)) == NULL
	) {	Throw_Nlog_error(errno); /* Error: insufficient memory */	}

	/* set `n' to one */
	if ((n = malloc(2)) == NULL) {
		Throw_Nlog_error(errno); /* Error: insufficient memory */
	}
	n[0] = '1'; n[1] = '\0';

	/* `nlogs[j]
	 *		:= sigma{	(`nlogs[i]' * `ratio_of_num') / (1 + 2 * `n')
	 *					[1 <= `n']
	 *				}
	 */
	for (j = 1; ; i = j) {
		/* set `j' that is index for the result of this iteration */
		j	= (i + 1) % 2;

		/* `n' := `n' + 2 */
		n_numerals	= 0
		; numerals[n_numerals++]	= n
		; numerals[n_numerals++]	= "2"
		;
		if ((a = aggregate_numbers(Aggregate_e_opcode_add
									, numerals
									, n_numerals
									, -1
									)) == NULL
		) {	Throw_Nlog_error(errno); /* Error: insufficient memory */	}
		(void)destroy_aggregate_numbers(n)
		; n	= a; a	= NULL
		;

		/* `power_of_ratio' := `power_of_ratio' * `ratio_of_num' */
		n_numerals	= 0
		; numerals[n_numerals++]	= power_of_ratio
		; numerals[n_numerals++]	= ratio_of_num
		;
		if ((a = aggregate_numbers(Aggregate_e_opcode_mul
									, numerals
									, n_numerals
									, -1
									)) == NULL
		) {	Throw_Nlog_error(errno); /* Error: insufficient memory */	}
		(void)destroy_aggregate_numbers(power_of_ratio)
		; power_of_ratio	= a; a	= NULL
		;

		/* `a' := `power_of_ratio' / `n' */
		n_numerals	= 0
		; numerals[n_numerals++]	= power_of_ratio
		; numerals[n_numerals++]	= n
		;
		if ((a = aggregate_numbers(Aggregate_e_opcode_div
												, numerals
												, n_numerals
												, div_scale
												)) == NULL
		) {	Throw_Nlog_error(errno); /* Error: insufficient memory */	}
		if (make_number(a, &x) == NULL) {
			Throw_Nlog_error(errno); /* Error: insufficient memory */
		}
		if (is_zero(&x) != 0) {
			(void)destroy_aggregate_numbers(a); a = NULL
			;
			break; /* we got the result `nlog[i]' */
		}

		/* `nlogs[j] := `nlogs[i]' + `a' */
		if (nlogs[j] != NULL) {
			(void)destroy_aggregate_numbers(nlogs[j])
			; nlogs[j]	= NULL
			;
		}
		n_numerals	= 0
		; numerals[n_numerals++]	= nlogs[i]
		; numerals[n_numerals++]	= a
		;
		if ((nlogs[j] = aggregate_numbers(Aggregate_e_opcode_add
												, numerals
												, n_numerals
												, -1
												)) == NULL
		) {	Throw_Nlog_error(errno); /* Error: insufficient memory */	}
		(void)destroy_aggregate_numbers(a); a = NULL;
	}

	/* release the working memories */
	if (ratio_of_num != NULL) {
		(void)destroy_aggregate_numbers(ratio_of_num);
	}
	if (nlogs[j] != NULL) {	(void)destroy_aggregate_numbers(nlogs[j]);	}
	if (power_of_ratio != NULL) {
		(void)destroy_aggregate_numbers(power_of_ratio);
	}
	if (n != NULL) {	(void)destroy_aggregate_numbers(n);	}

	/* `nlogs[i]' := 2 * `nlogs[i]' */
	n_numerals	= 0
	; numerals[n_numerals++]	= "2"
	; numerals[n_numerals++]	= nlogs[i]
	; if ((a = aggregate_numbers(Aggregate_e_opcode_mul
								, numerals
								, n_numerals
								, -1
								)) == NULL
	) {	Throw_Nlog_error(errno); /* Error: insufficient memory */	}
	(void)destroy_aggregate_numbers(nlogs[i])
	; nlogs[i]	= a; a	= NULL
	;

	return nlogs[i]; /* return normally */
#	undef	Throw_Nlog_error
}

/* API: Napierian logarithm */
char *ln_numbers(const char	*num , ssize_t		scale)
{
#	define		Throw_Ln_error(ERRNO)	{ lineno = __LINE__	\
										; errno = (ERRNO)	\
										; goto Ln_error		\
										;					\
										}
	const char	*numerals[2];
	Number_t	x;
	ssize_t		nlog_scale;
	size_t		n_numerals;
	char		*ln_of_10 = NULL, *p = NULL, *m = NULL, *r = NULL
				, *n = NULL, *a = NULL
				;
	int			cmp_result = 0, lineno = __LINE__, sv_errno = errno;

	/* test: arguments is valid? */
	if ((num == NULL) || make_number(num, &x) == NULL) {
		Throw_Ln_error(EINVAL); /* Error: invalid arguments */

Ln_error:	{ /* occurred several errors */
			if (p != NULL) { (void)destroy_aggregate_numbers(p);	}
			if (m != NULL) { (void)destroy_aggregate_numbers(m);	}
			if (ln_of_10 != NULL) {
				(void)destroy_aggregate_numbers(ln_of_10);
			}
			if (r != NULL) { (void)destroy_aggregate_numbers(r);	}
			if (n != NULL) { (void)destroy_aggregate_numbers(n);	}
			return	NULL; /* returns error */
		}
	}
	if (compare_numbers(num, "0", &cmp_result) == NULL) {
		Throw_Ln_error(errno); /* Error: invalid arguments */
	}
	else if (cmp_result <= 0) {
		Throw_Ln_error(EINVAL); /* Error: invalid arguments */
	}

	/* make `m' and `ln_of_10' from `num' */
	if (compare_numbers(num, "1", &cmp_result) == NULL) {
		Throw_Ln_error(EFAULT); /* FATAL (never!) */
	}
	else if (cmp_result < 0) {
		/* `num' is less than one */
		for (a = (char *)(x.dp + 1), nlog_scale = 0
		; (a < x.ep) && *a == '0'
		; ++nlog_scale, ++a
		) {	; /* count up leading zero sequence of fraction part */	}
		if ((p = malloc((MAX_LEN_OF_MAX_BIT_WIDTH_INTEGER + 1)))
															== NULL
		) {	Throw_Ln_error(errno); /* Error: insufficient memory */	}
		; (void)snprintf(p, (MAX_LEN_OF_MAX_BIT_WIDTH_INTEGER + 1)
						, "%lld", (long long)(-nlog_scale)
						)
		;

		if ((m = shift_numbers(num, -nlog_scale)) == NULL) {
			Throw_Ln_error(errno); /* Error: insufficient memory */
		}

		if (nlog_scale < scale) {	nlog_scale = scale;	}
	}
	else if(compare_numbers(num, "10", &cmp_result) == NULL) {
		Throw_Ln_error(EFAULT); /* FATAL (never!) */
	}
	else if (cmp_result > 0) {
		/* `num' is greater than ten */
		nlog_scale	= (ssize_t)((x.dp - x.ip) - 1);
		if ((p = malloc((MAX_LEN_OF_MAX_BIT_WIDTH_INTEGER + 1)))
															== NULL
		) {	Throw_Ln_error(errno); /* Error: insufficient memory */	}
		; (void)snprintf(p, (MAX_LEN_OF_MAX_BIT_WIDTH_INTEGER + 1)
						, "%lld", (long long)(nlog_scale)
						)
		;

		if ((m = shift_numbers(num, -nlog_scale)) == NULL) {
			Throw_Ln_error(errno); /* Error: insufficient memory */
		}

		nlog_scale	+= ((scale > 0)? scale: 0);
	}
	else { /* 1 <= `num' <= 10 */
		p	= NULL; 
		if ((m = duplicate_numbers(num)) == NULL) {
			Throw_Ln_error(errno); /* Error: insuficient memory */
		}
		ln_of_10	= NULL;
		nlog_scale	= ((scale > 0)? scale: 0);
	}
	if (p != NULL) {
		if ((a = (((size_t)nlog_scale < MAX_SCALE_OF_ln_of_10tbl)?
								lookup_ln_of_10tbl(nlog_scale)
					:			napierian_logarithm("10", nlog_scale)
					)) == NULL) {
			Throw_Ln_error(errno); /* Error: insuficient memory */
		}
		n_numerals	= 0
		; numerals[n_numerals++]	= p
		; numerals[n_numerals++]	= a
		;
		if ((ln_of_10 = aggregate_numbers(Aggregate_e_opcode_mul
											, numerals
											, n_numerals
											, -1
											)) == NULL
		) {	Throw_Ln_error(errno); /* Error: insuficient memory */	}
		(void)destroy_aggregate_numbers(p);		p	= NULL
		; (void)destroy_aggregate_numbers(a);	a	= NULL
		;
	}

	/* make `r' using `logl(3M) (in math-library "libm.so")' */
	{
		long double	r_val;
		char		*endptr = m;

		errno	= 0
		; r_val	= strtold(m, &endptr)
		;
		if (errno != 0) {
			/* Error: ERANGE (over flow or under flow) */
			Throw_Ln_error(errno);
		}
		r_val	= logl(r_val);
		if (errno != 0) {
			Throw_Ln_error(EFAULT); /* Error: FATAL (never!) */
		}

		if ((r = malloc(PRECISON_OF_LOGL_MATH_LIB + 3)) == NULL) {
			Throw_Ln_error(EFAULT); /* Error: FATAL (never!) */
		}
		(void)snprintf(r, (PRECISON_OF_LOGL_MATH_LIB + 3)
						, "%.*Lf", PRECISON_OF_LOGL_MATH_LIB, r_val
						);
	}

	/* `n' := `m' / exponent_numbers( `r' ) */
	if ((a = exponent_numbers(r, nlog_scale)) == NULL) {
		Throw_Ln_error(errno); /* Error: insufficient memory */
	}
	n_numerals	= 0
	; numerals[n_numerals++]	= m
	; numerals[n_numerals++]	= a
	;
	if ((n = aggregate_numbers(Aggregate_e_opcode_div
								, numerals
								, n_numerals
								, (nlog_scale * 2)
								)) == NULL
	){	Throw_Ln_error(errno); /* Error: insufficient memory */	}
	(void)destroy_aggregate_numbers(m);		m	= NULL
	; (void)destroy_aggregate_numbers(a);	a	= NULL
	;

	/* if `ln_of_10' isn't NULL, `r' := `r' + `ln_of_10' */
	if (ln_of_10 != NULL) {
		n_numerals	= 0
		; numerals[n_numerals++]	= r
		; numerals[n_numerals++]	= ln_of_10
		;
		if ((a = aggregate_numbers(Aggregate_e_opcode_add
									, numerals
									, n_numerals
									, -1
									)) == NULL
		){	Throw_Ln_error(errno); /* Error: insufficient memory */	}
		(void)destroy_aggregate_numbers(ln_of_10); ln_of_10	= NULL
		; (void)destroy_aggregate_numbers(r)
		; r	= a; a	= NULL
		;
	}

	/* `p' := napierian_logarithm( `n' ) + `r' */
	if ((a = napierian_logarithm(n, nlog_scale)) == NULL) {
		Throw_Ln_error(errno); /* Error: insufficient memory */
	}
	(void)destroy_aggregate_numbers(n);		n	= NULL
	; n_numerals	= 0
	; numerals[n_numerals++]	= a
	; numerals[n_numerals++]	= r
	; if ((p = aggregate_numbers(Aggregate_e_opcode_add
								, numerals
								, n_numerals
								, -1
								)) == NULL
	) {	Throw_Ln_error(errno); /* Error: insufficient memory */	}
	(void)destroy_aggregate_numbers(a);		a	= NULL
	; (void)destroy_aggregate_numbers(r);	r	= NULL
	;

	/* make the result from `p', according to `scale' */
	if ((r = round_numbers(Aggregate_e_round_down
							, (const char *)p
							, ((scale > 0)? (scale + 1): 0)
							)) == NULL
	) {	Throw_Ln_error(errno); /* Error: insufficient memory */	}
	(void)destroy_aggregate_numbers(p);		p	= NULL;

	errno	= sv_errno; return	r; /* return normally */
#	undef	Throw_Ln_error
}

/* API: release the buffer that is allocated by `aggregate_numbers()' */
int destroy_aggregate_numbers(char	*aggregate_numbers_result)
{
	int	sv_errno = errno;

	if (aggregate_numbers_result == NULL) {
		errno = EINVAL; return !0; /* Error: invalid arguments */
	}

	free(memset(aggregate_numbers_result
			, (int)'\0'
			, strlen(aggregate_numbers_result)
			));

	errno = sv_errno; return 0; /* normal end */
}
