/***********************************************************************
 * File: buffer_checker.h
 * (Extracted from
 * "$Id: buffer_checker.h,v 1.3 2009/05/24 11:52:28 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C header file defines the typedefs, macros and prototyping
 * for buffer_checker.c.
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#include <stddef.h>		/* for offsetof() macro */
#include <stdlib.h>		/* for standard libraries */
#include <ctype.h>		/* for string library */
#include <string.h>		/* for strcmp(3) */
#include <sys/types.h>	/* for regex(3) */
#include <regex.h>		/* for regex(3) */
#include <limits.h>		/* for INT_MIN */


#ifndef _BUFFER_CHECKER_H_ /* { */
#define _BUFFER_CHECKER_H_

/* Definitions the result of the following API-s */
typedef enum buffer_checker_result_t {
	/* Usually result of checking... */
	  BCR_Continue	= INT_MIN /* Normal End. User continues the checking */
	, BCR_Terminate			  /* Normal End. User finishes the checking */
	, BCR_Unmatched			  /* Abnormal End. User fails the checking */

	/* Failed in checking procedures or illegal definitions... */
	, BCR_Illegal_args					/* Illegal arguments */
	, BCR_Illegal_defines				/* Illegal specifications */
	, BCR_Cannot_allocate_preg			/* Can't allocate memory */
	, BCR_Failed_to_compile_regex		/* Failed compiling regex */
	, BCR_Not_initialized_defines		/* Calling sequence error */
	, BCR_Cannot_get_field_addr			/* Can't get field address */
	, BCR_Field_runs_over_buffer		/* Field runs over the buffer */
	, BCR_Field_size_is_illegal			/* Field size is illegal */
	, BCR_Field_string_is_too_long		/* Field string is too long */
	, BCR_Broken_Configuration			/* Broken configuration */
	, BCR_Out_of_range_cond_cb_returns	/* `cond_cb' callback returns
										 * out of range for
										 * `clause_cbs[]' array index
										 */
} Buffer_checker_result_t;


/* Type definitions; this enumerate defines `regex_result' arg
 * for `cond_cb' callback
 */
typedef enum buffer_checker_regex_result_t {
	  BCRR_Matched = 0
	, BCRR_Unmatched
	, BCRR_Not_Specified_Regex
} Buffer_checker_regex_result_t;


/* Type definition; this structure describes the field and its regex */
typedef struct buffer_checker_t {
	ssize_t	fld_offset;	/* offset of a field from the buffer address */

	size_t	fldsz;		/* size of field */
#	define Buffer_checker_fld(S,F) offsetof(S, F), sizeof(((S *)NULL)->F)

	void	*(* user_get_fld_addr)(void	 	*buffer
								, ssize_t	fld_offset
								, size_t	*fldszp
								, void	 	*user_cntxt
								);	/* If it isn't NULL,
									 * `exec_buffer_checker()' invoke this
									 * to get the address of the field.
									 * `fldszp' points to the size variable
									 * (if  `fldsz' member was set, the
									 *  variable would be set as it before
									 *  invoking).
									 */

	char	*regex;		/* regular expression for the field */

	regex_t	*preg;		/* holds the compiled regular expressions */

	ssize_t (*cond_cb)(size_t	 					checker_index
					, void							*buffer
					, size_t	 					buffer_sz
					, void							*fldp
					, Buffer_checker_regex_result_t	regex_result
					, void							*user_cntxt
					);	/* pointing to function that is callback for
						 * checking the field.  The callback function must
						 * return the index of the following `clause_cbs[]'
						 * array. The range of result is between 0 to 2.
						 */

	int (*clause_cbs[3])(size_t	 	checker_index
						, void		*buffer
						, size_t	 buffer_sz
						, void		*fldp
						, void		*user_cntxt
						);	/* array of pointer to functions those are
							 * callbacks.  The each functions must return
							 * Buffer_check_result_t, zero or positive
							 * integer (zero or positives means the next
							 * index of the `Buffer_checker_t' array).
							 *
							 * `check_index' indicates the index of the
							 * array of Buffer_checker_t.  `buffer' is
							 * pointing to the buffer of fields.
							 * `buffer_sz' indicates the size of `buffer'.
							 * `fldp' points to address the field.
							 * `user_cntxt' is pointing to the optional
							 * data.
							 *
							 * If `cond_cb' callback pointer is NULL, the
							 * field matches the regular expression will be
							 * deliver to the user's callback
							 * `clause_cbs[0]' pointing to.
							 * The field doesn't match will be deliver to
							 * the the user's callback `clause_cbs[1]'
							 * pointing to.
							 *
							 * If 'cond_cb' callback pointer isn't NULL
							 * (the field matches regular expression), the
							 * field will be deliver to the entry of
							 * `clause_cbs[]' indexed by the result from
							 * `cond_cb' callback function.
							 */

#	define Buffer_checker_RE(RE,MATCHED_CB,UNMATCHED_CB)				\
					(RE), NULL, NULL, { (MATCHED_CB), (UNMATCHED_CB), NULL }
#	define Buffer_checker_ANY_MATCH_RE(MATCHED_CB)			\
					Buffer_checker_RE("^.*$",(MATCHED_CB),NULL)
#	define Buffer_checker_COND(RE,COND_CB,NEG_CB,EQ_CB,POSI_CB) 		\
					(RE), NULL, (COND_CB), { (NEG_CB), (EQ_CB), (POSI_CB) }
} Buffer_checker_t;

/* declare a list of regular-expression and processed-expression pairs. */
typedef struct regex_preg_pair_t {
	char	*regex;	/* pointing to a regular expression */
	regex_t	*preg;	/* pointing to a processed expression
					 * that is allocated memory.
					 */
} Regex_preg_pair_t; /* for `pairs' of `struct regexe_preg_list_t' */

typedef struct regex_preg_list_t {
	Regex_preg_pair_t	*pairs;
	ssize_t			 	pairs_cnt;
} Regex_preg_list_t;

/* API: Initialize the `Buffer_checker_t' structure array.
 *	This API must be invoked once, at the first in the main thread.
 */
extern Buffer_checker_result_t init_buffer_checker(
										  Buffer_checker_t	*defines
										, size_t	    	defines_cnt
										, size_t	    	buffer_sz
										, Regex_preg_list_t	*regex_preg_list
										);

/* API: Check the all fields in a buffer, using initialized array. */
extern Buffer_checker_result_t exec_buffer_checker(
										  void	   			*buffer
										, size_t	    	buffer_sz
										, Buffer_checker_t	*defines
										, size_t	    	defines_cnt
										, void		   		*user_cntxt
										, size_t	   		*executed_cnt
										);

/* API: Clean up the `Buffer_checker_t' structure array.
 *	This API must be invoked once, at last in the main thread.
 */
extern Buffer_checker_result_t finish_buffer_checker(
										  Buffer_checker_t	*defines
										, ssize_t	    	defines_cnt
										, Regex_preg_list_t	*regex_preg_list
										);

/* API: For debugging, dump the `regex_preg_list' structure.
 *      `debug' argument is not zero, this API prints it to stderr.
 */
extern void dump_buffer_checker_regex_preg_list(
										  int debug
										, Regex_preg_list_t	*regex_preg_list
										);

#endif /* !defined(_BUFFER_CHECKER_H_) } */
