/***********************************************************************
 * File: fair_copy.c
 * 	(Extracted from
 *	   "$Id: fair_copy.c,v 1.3 2009/05/30 20:27:19 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C source implements the `fair_copy()' API.
 *
 * Copyright (C) 2004-2007 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#include "fair_copy.h" /* definitions for `fair_copy()' API */


/* translate new-line code to "\n", and copy it to a string, and
 * substitute space codes (without space ' ' and line feed '\n') to space
 */
static char *chnl_strncpy(char *dst, char *str, size_t n)
{
	char	*result	= dst;

	while ((n-- > 0) && *str != '\0') {
		if ((((isspace(*str))
		&& *str != ' ') && *str != '\r') && *str != '\n'
		) {	*dst++	= ' '; ++str;	}
		else if (*str == '\r') {
			if (*(str + 1) == '\n') { /* DOS style */
				str	+= 2;
			}
			else { /* MacOS style */
				++str;
			}
			*dst++	= '\n';
		}
		else { /* Unix style or non new-line code */
			*dst++	= *str++;
		}
	}

	*dst	= '\0'; /* put end-of-string */

	return	result;
}


/*
 * This API processes the source string to fit the rows X cols box area.
 *
 * arguments:
 *	src is pointing to the source string.
 *	linectl indicates that text-rectangle needs the new-line codes
 *	for the each end-of-line (Fair_copy_line_breaking) or not
 *	(Fair_copy_wrap_around).
 *	rows indicates the number of the rows of the box area.
 *	cols indicates the number of columns of the box area.
 *	dst_sz indicates the size of the destiny string.
 *	dst is pointing to the destiny (outputs processed string).
 *	(If dst is NULL, this API outputs processed_rows, only.)
 *	processed_rows outputs the rows of dst string.
 *
 * returns:
 *	Zero:	Normal end
 *	1	:	Illegal argument
 *	2	:	Failed to allocate the duplicated source string
 *	3	:	Rows of output is greater than 'rows'
 *	4	:	Output is overflow the destiny string size.
 */
int fair_copy(char						*src
			, Fair_copy_line_control	linectl
			, size_t					rows
			, size_t					cols
			, size_t					dst_sz
			, char						*dst
			, size_t					*processed_rows
			)
{
	size_t	left_len, dsz = 0L;
	char	*dup_src, *sep;
	int		retcode = FAIR_COPY_NORMAL_END, start_of_para = !0;

	/* test: illegal arguments? */
	if ((((src == NULL) || cols == 0)
	|| processed_rows == NULL) || ((dst != NULL) && dst_sz == 0)
	) {	return	FAIR_COPY_ILLEGAL_ARGS;	}

	/* duplicate the source string */
	left_len	= strlen(src);
	if ((dup_src = (char *)malloc(left_len + 1)) == NULL) {
		return	FAIR_COPY_NO_MEMORY;
	}
	src	= chnl_strncpy(dup_src, src, left_len);

	/* trim the tail space and line feeds */
	for (sep = src + (strlen(src) - 1)
	; (sep >= src) && (*sep == ' ' || *sep == '\n')
	; --sep
	) {	*sep	= '\0';	}

	/* test: the src is empty string? */
	if (*src == '\0') {
		if (dst != NULL) {	*dst	= '\0';	}
		*processed_rows = 1;
		return	retcode; /* normal end */
	}
	left_len	= strlen(src);

	/* process the duplicated string... */
	if (dst != NULL) {
		memset(dst, (int)'\0', dst_sz); /* initialize destiny buf */
	}
	*processed_rows	= 0L;
	while ((*src != '\0')
	&& ((dst == NULL) || *processed_rows < rows)
	) {
		size_t	len;

		/* test: line feed code is the first? */
		if (*src == '\n') {
			/* put the empty line */
			++(*processed_rows);
			if (dst != NULL) {
				if (*processed_rows >= rows) {
					*dst++	= ' ';
					retcode	= FAIR_COPY_ROWS_OVER;
					break; /* ERROR */
				}
				*dst++	= '\n';
				--dst_sz;
			}
			++dsz;
			++src; --left_len;
			start_of_para	= !0;
		}
		else { /* process a sentence... */
			char	*spcp, *pspcp, *lfp;

			/* procedure to put the sentence/sub-string */
#			define COPY_LETTERS(SEP) {								\
				if (len > dst_sz - 1) {								\
					len	= dst_sz - 1;								\
				}													\
				memcpy(dst, src, len);								\
				dst += len;											\
				if (((*(SEP) != '\0') && *processed_rows >= rows)	\
				|| len < (size_t)((SEP) - src)						\
				) {													\
					retcode	= FAIR_COPY_ROWS_OVER;					\
					break; /* ERROR */								\
				}													\
			}

			/* get end of sentence (sep) */
			sep	= src + ((left_len > cols)? cols: left_len);

			/* get addresses of the last space code and the last code of
			 * the word exists before the space sequence
			 */
			if (cols > 1) {
				for (spcp = sep - 1
				; (spcp >= src) && *spcp != ' '
				; --spcp
				) {	;	}
				for (pspcp = spcp - 1
				; (pspcp >= src) && *pspcp == ' '
				; --pspcp
				) {	;	}
			}
			else { /* cols is one character width */
				pspcp = spcp = src - 1;
			}

			/* test: the line feed exists in the sentence?  */
			for (lfp = src
			; (lfp < sep) && *lfp != '\n'
			; ++lfp
			) {	;	}
			if (lfp < sep) {
				/* split the sentence at the line feed */
				++(*processed_rows);
				len	= (size_t)(lfp - src);
				if (dst != NULL) {
					COPY_LETTERS(lfp);

					*dst++ = '\n'; dst_sz -= (len + 1);
				}
				++len; dsz	-= len; src += len; left_len -= len;
				start_of_para = !0;
			}
			else if ((*sep == '\n') || *sep == ' ') {
				/* the last word means the end of the sentence
				 * (*sep == '\n'), or the word strides the rows
				 * (*sep == ' ').
				 */
				++(*processed_rows);
				len	= (size_t)(sep - src);
				if (dst != NULL) {
					COPY_LETTERS(sep);

					/* if the line-control direction means Line-Breaking,
					 * put a line feed
					 */
					if (linectl == Fair_copy_line_breaking) {
						*dst++ = '\n'; dst_sz -= (len + 1);
					}
					else {
						dst_sz	-= len;
					}
				}
				dsz	+= (len + ((Fair_copy_line_breaking == linectl)? 1: 0));
				if (*sep == '\n') {
					++len;
				}
				src += len; left_len -= len;
				start_of_para	= (*sep == '\n')? !0: 0;
			}
			else if ((*sep != '\0') && pspcp >= src) {
				/* (except the last word) the sentence has a line feed code
				 */
				++(*processed_rows);
				len	= (size_t)((pspcp + 1) - src);
				if (dst != NULL) {
					COPY_LETTERS(pspcp + 1);

					*dst++ = '\n'; dst_sz -= (len + 1);
				}
				dsz	+= len + 1;
				len	= (size_t)((spcp + 1) - src);
				src += len; left_len -= len;
				start_of_para	= 0;
			}
			else {	/* put the sentence */
				++(*processed_rows);
				len	= (size_t)(sep - src);
				if (dst != NULL) {
					COPY_LETTERS(sep);

					/* (except the last word) the line control direction
					 * means Line-Breaking, put the line feed
					 */
					if ((*sep != '\0')
					&& linectl == Fair_copy_line_breaking
					) {
						*dst++ = '\n'; dst_sz -= (len + 1);
					}
					else {
						dst_sz	-= len;
					}
				}
				dsz	+= (len + ((Fair_copy_line_breaking == linectl)? 1: 0));
				src += len; left_len -= len;
				start_of_para	= 0;
			}
#			undef COPY_LETTERS
		}

		/* get address of the next sentence */
		if ((cols == 1) || start_of_para != 0) {
			/* width is one character, or the next sentence means
			 * the start of paragraph
			 */
			for (sep = src
			; (*sep != '\0') && *sep == ' '
			; ++sep
			) {	; /* search the first non-space code */	}
			len	= (size_t)(sep - src);
			if (len > 0) { /* the lead-off spaces exists */
				if (*sep == '\n') {
					/* the sentence means empty row */
					src = sep; left_len -= len;
				}
				else {	/* skip unnecessary spaces */
					len =  (((len / cols) - (((cols > 1)
											&& len % cols != 0)? 0: 1
											)
								) * cols
							);
					src += len; left_len -= len;
				}
			}
		}
		else {	/* skip unnecessary spaces */
			while ((*src != '\0') && *src == ' ') {
				++src; --left_len;
			}
		}
	}

	free(dup_src); dup_src = NULL; /* release the allocated buf */

	return	retcode; /* normal end */
}
