/***********************************************************************
 * File: sort_linked_list.c
 * (Extracted from
 * "$Id: sort_linked_list.c,v 1.3 2009/05/30 22:45:58 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This is the C source file that implements the 'sorting linked list'
 *  utility API.
 *
 * Copyright (C) 2005-2007 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/
#include "sort_linked_list.h"

/*
 * Abstracts;
 *	The "sort_linked_list()" API implements the sorting utility for
 * unidirectional and bidirectional linked lists, using "qsort()".
 * If the linked list was sorted completely, this returns the address of
 * the first node of the sorted.
 *
 * Arguments:
 *	  'list' points to the linked list.
 *	  'nodes_n' indicates the number of nodes in the linked list.
 *	If the negative integer is specified, it means the number is unknown.
 *	  'next' is the address of function that must return the address
 *	of the next node.
 *	  'prev (OPTIONAL)' is the address of function that must return
 *	the address of the previous node.
 *	  'get_datum_addr (OPTIONAL)' is the address of function that
 *	must return the address of the datum that must be pointed from
 *	the node.
 *	  'get_datum_fld_addr_on_node (OPTIONAL)' is the address of
 *	function that must return the address of the member of node,
 *	the member must be pointing to the datum.
 *	  'cmp_nodes_via_ptr2ptr' is the address of  function that
 *	compares two nodes or datums those are pointed by the
 *	pointer-to-pointer.
 *	  'set_next' is the address of function that sets the next node
 *	address to the node's 'next' member.
 *	  'set_prev (OPTIONAL)' is the address of function that sets the
 *	previous node address to the node's 'previous' member.
 *	  'indexp (OPTIONAL)' is the pointer to array of the nodes that
 *	is ordered as the nodes have been sorted.  If it is set NULL,
 *	this API will allocate the memory for array and output the
 *	allocated address to this.  The allocated are must be
 *	'free()'-ed by invokers.
 *
 * If several errors occurred, this returns NULL and set 'errno'.
 *	'errno'	: means...
 *	EINVAL	: illegal arguments
 *	ENOMEM	: insufficient memory
 */
void *sort_linked_list(void		*list
					, ssize_t	nodes_n
					, void		*(*next)(const void *list)
					, void		*(*prev)(const void *list)
					, void		*(*get_datum_addr)(const void *node)
					, void		*(*get_datum_fld_addr_on_node)(
															const void *node
															)
					, int		(*cmp_nodes_via_ptr2ptr)(const void *node1
														, const void *node2
														)
					, void		*(*set_next)(void *src_node
											, const void *next_node
											)
					, void		*(*set_prev)(void *src_node
											, const void *prev_node
											)
					, void		***indexp
					)
{
	ssize_t	i, index_is_specified;
	void	*lp, *tail, *prev_of_head, *next_of_tail
			, **saved_vector = NULL, **vector = NULL
			;

	errno	= 0; /* initialize errno */

	/* test the arguments are illegal? */
	if (((((((list == NULL)
	|| next == NULL)
	|| cmp_nodes_via_ptr2ptr == NULL)
	|| (get_datum_addr != NULL && get_datum_fld_addr_on_node == NULL))
	|| (get_datum_addr == NULL && ((get_datum_fld_addr_on_node != NULL)
									|| set_next == NULL
									)))
	|| (prev != NULL && set_prev == NULL))
	|| (prev == NULL && set_prev != NULL)
	) {	errno = EINVAL; return NULL; /* illegal arguments */	}

	/* save the address that set to vector */
	saved_vector = vector = (((index_is_specified = (indexp != NULL)) == 0)?
															NULL: *indexp
							);

	/* test number of nodes is unknown? */
	if (nodes_n < 0L) { /* count up the nodes in the linked list */
		for (nodes_n = 0UL, lp = list; lp != NULL; lp = (*next)(lp)) {
			++nodes_n;
		}
	}

	/* test vector was set to NULL?  After then, failed to allocate
	 * the vector of the node or datum addresses?
	 */
	if (((vector == NULL) || get_datum_addr != NULL)
	&& (vector = (void **)malloc(sizeof(*vector) * nodes_n)) == NULL
	) {
		errno = ENOMEM; return NULL; /* insufficient memories */
	}
	memset(vector, (int)'\0', sizeof(*vector) * nodes_n);

	/* initialize the vector */
	/* test: sort through the pointers of datum in the linked list? */
	if (get_datum_addr != NULL) { /* set the address of the datum */
		prev_of_head = next_of_tail = tail	= NULL;
		for (i = 0, lp = list
		; (i < nodes_n) && lp != NULL
		; lp = (*next)(lp), ++i
		) {	vector[i]	= (*get_datum_addr)(lp);	}
	}
	else {	/* set the address of nodes */
		prev_of_head	= ((prev == NULL)? NULL: (*prev)(list));
		for (i = 0, lp = list
		; (i < nodes_n) && lp != NULL
		; lp = (*next)(lp), ++i
		) {	vector[i]	= lp;	}
		tail			= vector[nodes_n - 1];
		next_of_tail	= (*next)(tail);
	}

	/* test: the number of nodes in the linked list is greater than 1? */
	if (nodes_n > 1) {
		/* sort the vector using 'qsort() (quick-sort algorithm)' */
		qsort(vector, nodes_n, sizeof(*vector), cmp_nodes_via_ptr2ptr);
	}

	/* test the vector has datum addresses? */
	if (get_datum_fld_addr_on_node != NULL) {
		/* test the saved vector was set to NULL? */
		if ((index_is_specified != 0) && saved_vector == NULL) {
			/* replace the datum member according to the sorted vector,
			 * and then, set the address of nodes to vector
			 */
			for (i = 0, lp = list
			; (i < nodes_n) && lp != NULL
			; lp = (*next)(lp), ++i
			) {
				*((char **)((*get_datum_fld_addr_on_node)(lp)))
												= ((char *)(vector[i]));
				vector[i]	= lp;
			}

			/* output the allocated area address to indexp */
			*indexp	= vector;
		}
		else { /* the sequence of nodes is never changed */
			/* replace the datum member according to the sorted vector */
			for (i = 0, lp = list
			; (i < nodes_n) && lp != NULL
			; lp = (*next)(lp), ++i
			) {
				*((char **)((*get_datum_fld_addr_on_node)(lp)))
												= ((char *)(vector[i]));
			}

			/* test the indexp is specified by invoker? */
			if (index_is_specified != 0) {
				*indexp = saved_vector; /* restore the indexp */
			}
			else { /* free the allocated area of datum addresses */
				free(memset(vector, (int)'\0', (sizeof(*vector) * nodes_n)))
				; vector = NULL
				;
			}
		}

		/* set the result to the first node of the original linked list */
		lp	= list;
	}
	else {	/* reconfigure the linked list according to the sorted vector */
		/* test the linked list has 'previous' member? */
		if (set_prev != NULL) {
			/* set the previous of the first node */
			(void)(*set_prev)(vector[0]
							, ((prev_of_head == tail)?	vector[nodes_n - 1]
								:						prev_of_head
								)
							);
		}

		/* link the nodes... */
		i = 0, lp = vector[0]; /* initialize the loop control variables */
		if (set_prev != NULL) {
			/* the linked list has 'previous' members */
			for ( ; i++ < nodes_n - 1; lp = vector[i]) {
				/* make this node to have the next */
				(void)(*set_next)(lp, vector[i]);

				/* make this node to have the previous */
				(void)(*set_prev)(vector[i], lp);
			}
		}
		else {	/* the linked list has no 'previous' members */
			for ( ; i++ < nodes_n - 1; lp = vector[i]) {
				/* make this node to have the next */
				(void)(*set_next)(lp, vector[i]);
			}
		}

		/* set the next of the last node */
		(void)(*set_next)(lp
						, ((next_of_tail == list)? vector[0]: next_of_tail)
						);

		/* set the result to the first node of the reconfigured linked list
		 */
		lp	= vector[0];

		/* test: the indexp isn't specified by invoker? */
		if (index_is_specified == 0) {
			/* release the allocated node addresses */
			free(memset(vector, (int)'\0', (sizeof(*vector) * nodes_n)))
			; vector = NULL
			;
		}
		else { /* output the allocated node address to indexp */
			*indexp	= vector;
		}
	}

	return	lp; /* returns the result pointing to the first node */
}
